package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleAdminOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticlePreviewOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.ArticleService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/content")
@RequiredArgsConstructor
public class ArticleController {

    private final ArticleService articleService;
    private final UserService userService;

    @GetMapping("/article/{id}")
    public ResponseEntity<ServerResponse<ArticleOut>> getArticle(
            @PathVariable String id,
            @RequestHeader(value = HttpHeaders.AUTHORIZATION, required = false) String token,
            HttpServletRequest request){
        String userEmail = null;
        if(token != null)
            userEmail = JwtUtil.extractUsernameFromHeader(token);
        return ServerResponse.createResponseEntity(articleService.getArticle(Integer.valueOf(id), userEmail, request.getRemoteAddr()));
    }

    @GetMapping("/article/{id}/image")
    public ResponseEntity<byte[]> getArticleImage(@PathVariable Integer id) throws IOException {
        return articleService.getArticleImage(id);
    }

    @GetMapping("/article/list")
    public ResponseEntity<ServerResponse<List<ArticlePreviewOut>>> getArticleList(){
        return ServerResponse.createResponseEntity(articleService.getArticleList(null));
    }

    @GetMapping("/article/list/{page}")
    public ResponseEntity<ServerResponse<List<ArticlePreviewOut>>> getArticleListByPage(@PathVariable String page){
        return ServerResponse.createResponseEntity(articleService.getArticleList(Integer.valueOf(page)));
    }

    @GetMapping("/article/search")
    public ResponseEntity<ServerResponse<List<ArticlePreviewOut>>> getArticlesSearch(@RequestParam String search){
        if(search == null || search.isEmpty() || search.isBlank())
            return ServerResponse.createResponseEntity(ServiceResult.createServiceResult(null, HttpStatus.NO_CONTENT));
        return ServerResponse.createResponseEntity(articleService.getArticleSearch(search));
    }

    @Secured("ROLE_ADMIN")
    @PostMapping("/article/")
    public ResponseEntity<ServerResponse<String>> addArticle(
            @RequestPart("articleData") ArticleIn articleIn,
            @RequestPart(value = "articleImage", required = false) MultipartFile image,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){

        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(
                articleService.addArticle(userId, articleIn, image));
    }

    @Secured("ROLE_ADMIN")
    @PutMapping("/article/{articleId}")
    public ResponseEntity<ServerResponse<String>> editArticle(
            @PathVariable Integer articleId,
            @RequestPart("articleData") ArticleIn articleIn,
            @RequestPart(value = "articleImage", required = false) MultipartFile image,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){

        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(articleService.editArticle(userId, articleId, articleIn, image));
    }

    @Secured("ROLE_ADMIN")
    @PatchMapping("/article/{articleId}/visibility")
    public ResponseEntity<ServerResponse<String>> changeVisibility(
            @PathVariable Integer articleId,
            @RequestBody Boolean visibility,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(
                articleService.changeVisibility(userId, articleId, visibility));
    }

    @Secured("ROLE_ADMIN")
    @DeleteMapping("/article/{articleId}")
    public ResponseEntity<ServerResponse<String>> deleteArticle(
            @PathVariable Integer articleId,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(articleService.deleteArticle(userId, articleId));
    }

    @Secured("ROLE_ADMIN")
    @GetMapping("/admin/article")
    public ResponseEntity<ServerResponse<List<ArticleAdminOut>>> getAdminArticleList(@RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(articleService.getAdminArticleList(userId));
    }
}
