package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.constants.FilePaths;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleSurveyOptionIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleAdminOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticlePreviewOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleSurveyOptionOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurvey;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurveyOption;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleViewActivity;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.*;
import cz.uhk.fim.redakcnisystem.contentmanagement.validation.ArticleValidation;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.UserRoleEn;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import lombok.RequiredArgsConstructor;
import org.apache.commons.codec.digest.DigestUtils;
import org.modelmapper.ModelMapper;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.ZonedDateTime;
import java.util.*;

@Service
@RequiredArgsConstructor
public class ArticleServiceImpl implements ArticleService {

    private final ModelMapper mapper = new ModelMapper();

    private final ArticleRepository articleRepository;
    private final UserRepository userRepository;
    private final ArticleViewActivityRepository articleViewActivityRepository;
    private final ArticleSurveyRepository articleSurveyRepository;
    private final ArticleSurveyOptionRepository articleSurveyOptionRepository;
    private final ArticleSurveyVoteRepository articleSurveyVoteRepository;

    private static final Integer PAGE_SIZE = 10;

    @Override
    public ServiceResult<ArticleOut> getArticle(Integer id, String userEmail, String remoteAddress) {
        Optional<Article> article = articleRepository.findById(id);
        //article is not exists
        if (article.isEmpty())
            return Errors.articleNotFound();
        boolean isAdmin = false;
        Optional<User> userOptional = Optional.empty();
        if ((userEmail != null)) {
            userOptional = userRepository.findByEmail(userEmail);
            if (userOptional.isPresent())
                isAdmin = userOptional.get().getRoles().stream().findAny().filter(userRole -> userRole.getRole().equals(UserRoleEn.ADMIN)).isPresent();

        }
        // article is not visible
        if (!article.get().getVisibility() && !isAdmin)
            return Errors.articleIsNotVisible();
        /* save article view activity
         * Save when uniqueRecognitionId is not used last 1 hour (prevent refresh spam)
         * Save when user is logged, and didnt have view last 1 hour
         * */
        remoteAddress = DigestUtils.sha256Hex(remoteAddress);
        if (userOptional.isPresent()) {
            Optional<ArticleViewActivity> activityCheckOptional = articleViewActivityRepository
                    .findFirstByArticleAndArticleViewTimeBetweenAndUser(
                            article.get(),
                            ZonedDateTime.now().minusMinutes(60),
                            ZonedDateTime.now(),
                            userOptional.get());
            if (activityCheckOptional.isEmpty()) {
                ArticleViewActivity articleViewActivity = new ArticleViewActivity();
                articleViewActivity.setArticle(article.get());
                articleViewActivity.setArticleViewTime(ZonedDateTime.now());
                articleViewActivity.setUniqueRecognitionId(remoteAddress);
                articleViewActivity.setUser(userOptional.get());
                articleViewActivityRepository.save(articleViewActivity);
            }
        } else {
            Optional<ArticleViewActivity> activityCheckOptional = articleViewActivityRepository
                    .findFirstByArticleAndUniqueRecognitionIdAndArticleViewTimeBetween(
                            article.get(),
                            remoteAddress,
                            ZonedDateTime.now().minusHours(1),
                            ZonedDateTime.now());
            if (activityCheckOptional.isEmpty()) {
                ArticleViewActivity articleViewActivity = new ArticleViewActivity();
                articleViewActivity.setArticle(article.get());
                articleViewActivity.setArticleViewTime(ZonedDateTime.now());
                articleViewActivity.setUniqueRecognitionId(remoteAddress);
                articleViewActivityRepository.save(articleViewActivity);
            }
        }
        // return article
        ArticleOut articleOut = mapper.map(article.get(), ArticleOut.class);
        if (JsonFilesRepository.generalSettings().getDisplayNumberOfViewsArticle())
            articleOut.setViewsCount(article.get().getArticleViewActivities().size());
        if(articleOut.getSurvey() != null){
            if(articleOut.getSurvey().getSurveyVisibleResults()){
                int articleSurveyId = article.get().getArticleSurvey().getId();
                List<ArticleSurveyOptionOut> optionsOut = new ArrayList<>();
                article.get().getArticleSurvey().getSurveyOptions().forEach(option -> {
                    ArticleSurveyOptionOut temp = mapper.map(option, ArticleSurveyOptionOut.class);
                    temp.setCount(articleSurveyVoteRepository.countAllByArticleSurveyIdAndSurveyOptionId(
                            articleSurveyId,
                            option.getId()).orElse(0));
                    optionsOut.add(temp);
                });
                articleOut.getSurvey().setSurveyOptions(optionsOut);
            }
        }
        return ServiceResult.createServiceResult(articleOut, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<ArticlePreviewOut>> getArticleList(Integer page) {
        List<Article> articles = articleRepository.findAll(Sort.by("createTime"));
        articles.removeIf(article -> !article.getVisibility());

        if (page != null) {
            if (articles.size() >= PAGE_SIZE * page) {
                articles.subList(PAGE_SIZE * (page - 1), PAGE_SIZE * page);
            } else if (articles.size() >= PAGE_SIZE * (page - 1)) {
                articles.subList(PAGE_SIZE * (page - 1), articles.size());
            } else if (articles.size() < PAGE_SIZE * (page - 1)) {
                return ServiceResult.createServiceResult(
                        Collections.singletonList(
                                new ServiceError("out-of-bound", "Články na této stránce neexistují.")),
                        HttpStatus.NOT_FOUND);
            }
        }

        if (articles.isEmpty())
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("articles-not-found", "Články nebyly nalezeny.")),
                    HttpStatus.NOT_FOUND);
        List<ArticlePreviewOut> articlePreviewOuts = new ArrayList<>();
        articles.forEach(article -> articlePreviewOuts.add(mapper.map(article, ArticlePreviewOut.class)));
        return ServiceResult.createServiceResult(articlePreviewOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<ArticlePreviewOut>> getArticleSearch(String search) {
        List<ArticlePreviewOut> articlePreviewOuts = articleRepository.findAllArticlePreviewOutBySearch(search);
        if (articlePreviewOuts.isEmpty())
            return Errors.singletonError(
                    "articles-with-search-not-found",
                    "Nebyl nalezen žádný článek podle vyhledávání.",
                    HttpStatus.NOT_FOUND);

        return ServiceResult.createServiceResult(articlePreviewOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> addArticle(Integer userId, ArticleIn articleIn, MultipartFile image) {
        List<ServiceError> errors = new ArrayList<>();
        ArticleValidation.validateArticleIn(errors, articleIn);
        if (!errors.isEmpty())
            return ServiceResult.createServiceResult(errors, HttpStatus.NOT_ACCEPTABLE);
        Article article = new Article();
        article.mapDataFromArticleIn(articleIn);
        articleRepository.save(article);
        if(image != null) {
            try {
                saveArticleImage(article.getId(), image);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (articleIn.getSurvey() != null && !articleIn.getSurvey().getSurveyOptions().isEmpty()) {
            ArticleSurvey articleSurvey = new ArticleSurvey();
            articleSurvey.setArticle(article);
            articleSurvey.setSurveyText(articleIn.getSurvey().getSurveyText());
            articleSurvey.setSurveyVisibleResults(articleIn.getSurvey().getSurveyVisibleResults());
            articleSurveyRepository.save(articleSurvey);
            articleIn.getSurvey().getSurveyOptions().forEach(option -> {
                ArticleSurveyOption surveyOption = new ArticleSurveyOption();
                surveyOption.setArticleSurvey(articleSurvey);
                surveyOption.setText(option.getText());
                surveyOption.setPosition(option.getPosition());
                articleSurveyOptionRepository.save(surveyOption);
            });
        }
        return ServiceResult.createServiceResult("Uloženo.", HttpStatus.OK);
    }

    private void saveArticleImage(Integer id, MultipartFile image) throws IOException {
        if (!new File(FilePaths.ARTICLE_IMAGE_FOLDER).exists()) {
            Files.createDirectories(Path.of(FilePaths.ARTICLE_IMAGE_FOLDER));
        }
        String extension = image.getOriginalFilename().substring(image.getOriginalFilename().lastIndexOf(".") + 1);
        String name = "article-" + id + "." + extension;
        Arrays.stream(new File(FilePaths.ARTICLE_IMAGE_FOLDER).list()).forEach(fileName -> {
            if(fileName.startsWith("article-"+id+".")){
                new File(FilePaths.ARTICLE_IMAGE_FOLDER + fileName).delete();
                return;
            }
        });
        image.transferTo(Paths.get(FilePaths.ARTICLE_IMAGE_FOLDER + name));
    }

    @Override
    public ServiceResult<String> editArticle(Integer userId, Integer articleId, ArticleIn articleIn, MultipartFile image) {
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        //article exists
        if (articleOptional.isEmpty())
            return Errors.articleNotFound();
        List<ServiceError> errors = new ArrayList<>();
        ArticleValidation.validateArticleIn(errors, articleIn);
        if (!errors.isEmpty())
            return ServiceResult.createServiceResult(errors, HttpStatus.NOT_ACCEPTABLE);
        Article article = articleOptional.get();
        article.setTitle(articleIn.getTitle());
        article.setDescription(articleIn.getDescription());
        article.setText(articleIn.getText());
        article.setLastUpdateTime(ZonedDateTime.now());
        article.setVisibility(articleIn.getVisibility());
        articleRepository.save(article);
        if (articleIn.getSurvey() != null && !articleIn.getSurvey().getSurveyOptions().isEmpty()) {
            ArticleSurvey articleSurvey;
            if(articleOptional.get().getArticleSurvey() != null) {
                articleSurvey = articleOptional.get().getArticleSurvey();
            } else {
                articleSurvey = new ArticleSurvey();
                articleSurvey.setArticle(article);
            }
            articleSurvey.setSurveyText(articleIn.getSurvey().getSurveyText());
            articleSurvey.setSurveyVisibleResults(articleIn.getSurvey().getSurveyVisibleResults());
            articleSurveyRepository.save(articleSurvey);
            if(articleOptional.get().getArticleSurvey() != null) {
                int maxPos = Math.max(
                        articleSurvey.getSurveyOptions().get(articleSurvey.getSurveyOptions().size()-1).getPosition(),
                        articleIn.getSurvey().getSurveyOptions().get(articleIn.getSurvey().getSurveyOptions().size()-1).getPosition());
                int minPos = Math.min(
                        articleSurvey.getSurveyOptions().get(0).getPosition(),
                        articleIn.getSurvey().getSurveyOptions().get(0).getPosition());
                for (int i = minPos; i <= maxPos; i++) {
                    int finalI = i;
                    Optional<ArticleSurveyOptionIn> inOptional = articleIn.getSurvey().getSurveyOptions()
                            .stream().filter(opt -> opt.getPosition() == finalI).findFirst();
                    Optional<ArticleSurveyOption> actualOptional = articleSurvey.getSurveyOptions()
                            .stream().filter(opt -> opt.getPosition() == finalI).findFirst();
                    if(inOptional.isEmpty() && actualOptional.isEmpty())
                        continue;
                    if(inOptional.isEmpty() && actualOptional.isPresent()){
                        articleSurveyOptionRepository.delete(actualOptional.get());
                    }
                    if(inOptional.isPresent() && actualOptional.isEmpty()){
                        ArticleSurveyOption tempSave = new ArticleSurveyOption();
                        tempSave.setText(inOptional.get().getText());
                        tempSave.setPosition(inOptional.get().getPosition());
                        tempSave.setArticleSurvey(articleSurvey);
                        articleSurveyOptionRepository.save(tempSave);
                    }
                    if(inOptional.isPresent() && actualOptional.isPresent()) {
                        ArticleSurveyOption edit = actualOptional.get();
                        edit.setText(inOptional.get().getText());
                        articleSurveyOptionRepository.save(edit);
                    }
                }
            } else {
                articleIn.getSurvey().getSurveyOptions().forEach(option -> {
                    ArticleSurveyOption surveyOption = new ArticleSurveyOption();
                    surveyOption.setArticleSurvey(articleSurvey);
                    surveyOption.setText(option.getText());
                    surveyOption.setPosition(option.getPosition());
                    articleSurveyOptionRepository.save(surveyOption);
                });
            }
        }
        if(image != null) {
            try {
                saveArticleImage(articleId, image);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ServiceResult.createServiceResult("Změny uloženy.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<ArticleAdminOut>> getAdminArticleList(Integer userId) {
        List<ArticleAdminOut> adminOuts = articleRepository.findAllArticleAdminOut();
        adminOuts.forEach(adminOut -> adminOut.setViewCount(articleViewActivityRepository.countByArticleId(adminOut.getId())));
        return ServiceResult.createServiceResult(adminOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> changeVisibility(Integer userId, Integer articleId, Boolean visibility) {
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        if (articleOptional.isEmpty())
            return Errors.articleNotFound();

        articleOptional.get().setVisibility(visibility);
        articleRepository.save(articleOptional.get());
        return ServiceResult.createServiceResult("Změna provedena.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteArticle(Integer userId, Integer articleId) {
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        if (articleOptional.isEmpty())
            return Errors.articleNotFound();
        articleRepository.delete(articleOptional.get());
        if(new File(FilePaths.ARTICLE_IMAGE_FOLDER).exists()){
            Arrays.stream(new File(FilePaths.ARTICLE_IMAGE_FOLDER).list()).forEach(fileName -> {
                if(fileName.startsWith("article-"+articleId+".")){
                    new File(FilePaths.ARTICLE_IMAGE_FOLDER + fileName).delete();
                    return;
                }
            });
        }
        return ServiceResult.createServiceResult("Článek byl odebrán.", HttpStatus.OK);
    }

    @Override
    public ResponseEntity<byte[]> getArticleImage(Integer id) throws IOException {
        if(new File(FilePaths.ARTICLE_IMAGE_FOLDER).exists()){
            final String[] image = {null};
            Arrays.stream(Objects.requireNonNull(new File(FilePaths.ARTICLE_IMAGE_FOLDER).list())).forEach(nameFile ->{
                if(nameFile.startsWith("article-"+id+".")){
                    image[0] = new String(nameFile);
                    return;
                }
            });
            if(image[0] != null){
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.valueOf(Files.probeContentType(Path.of(FilePaths.ARTICLE_IMAGE_FOLDER + image[0]))));
                return new ResponseEntity<>(
                        Files.readAllBytes(Path.of(FilePaths.ARTICLE_IMAGE_FOLDER + image[0])),
                        headers,
                        HttpStatus.OK);
            }
        }
        try {
            InputStream in = getClass().getResourceAsStream("/images/default-profile-image.png");
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.IMAGE_PNG);
            return new ResponseEntity<>(in.readAllBytes(), headers, HttpStatus.OK);
        } catch(Exception e ){
            e.printStackTrace();
        }
        return new ResponseEntity<>(null, HttpStatus.NO_CONTENT);
    }
}
