package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.CommentArticleIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.CommentArticleForUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.CommentArticleOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.CommentArticle;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.CommentArticleRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.validation.ArticleCommentValidation;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import cz.uhk.fim.redakcnisystem.usermanagement.validation.UserValidation;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@RequiredArgsConstructor
public class CommentArticleServiceImpl implements CommentArticleService {

    private final ArticleRepository articleRepository;
    private final UserService userService;
    private final UserRepository userRepository;
    private final CommentArticleRepository commentArticleRepository;


    @Override
    public ServiceResult<String> addCommentToArticle(CommentArticleIn commentArticleIn, Integer userID) {
        // validate body of article comment
        List<ServiceError> errorList = new ArrayList<>();
        ArticleCommentValidation.validateComment(errorList, commentArticleIn);
        if(!errorList.isEmpty())
            return ServiceResult.createServiceResult(errorList, HttpStatus.NOT_ACCEPTABLE);
        //check if user is valid
        if(!userService.userIsValid(userID)) {
            return Errors.youMustLogIn();
        }
        //check if article is visible
        Optional<Article> articleOptional = articleRepository.findById(commentArticleIn.getArticleId());
        if(articleOptional.isPresent()){
            if(!articleOptional.get().getVisibility()) {
                return Errors.articleIsNotVisible();
            }
        } else {
            //return error article is not exists
            return Errors.articleNotFound();
        }
        // save
        CommentArticle commentArticle = new CommentArticle();
        commentArticle.setArticle(articleOptional.get());
        commentArticle.setComment(commentArticleIn.getComment());
        commentArticle.setUser(userRepository.findById(userID).get());
        commentArticleRepository.save(commentArticle);
        return ServiceResult.createServiceResult("Komentář byl úspěšně přidán.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<CommentArticleOut>> getArticleComments(Integer articleId) {
        List<CommentArticle> comments = commentArticleRepository.findCommentArticlesByArticleId(articleId);
        // ??? error handle
        ModelMapper mapper = new ModelMapper();
        List<CommentArticleOut> commentArticleOuts = new ArrayList<>();
        comments.forEach(commentArticle -> {
            CommentArticleOut tempOut = mapper.map(commentArticle, CommentArticleOut.class);
            Optional<User> optionalUser = userRepository.findById(tempOut.getUserId());
            optionalUser.ifPresent(user -> tempOut.setUsername(user.getUsername()));
            commentArticleOuts.add(tempOut);
        });
        return ServiceResult.createServiceResult(commentArticleOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteComment(Long commentId, Integer userId) {
        //get comment
        Optional<CommentArticle> commentArticle = commentArticleRepository.findById(commentId);
        if(commentArticle.isEmpty())
            return ServiceResult.createServiceResult(Collections.singletonList(
                    new ServiceError("comment-not-found","Tento komentář nebyl nalezen v databázi.")), HttpStatus.NOT_FOUND);
        //check owner
        if(!Objects.equals(commentArticle.get().getUser().getId(), userId)){
            return ServiceResult.createServiceResult(Collections.singletonList(
                    new ServiceError("user-is-not-comment-owner","Uživatel není autorem tohoto komentáře.")), HttpStatus.NOT_ACCEPTABLE);
        }
        //do delete
        commentArticleRepository.delete(commentArticle.get());
        return ServiceResult.createServiceResult("Komentář byl úspěšně odstraněn.",HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<CommentArticleForUserOut>> getArticleCommentsForUser(Integer userId, boolean isLogged, String loggedEmail) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        //privacy check
        List<ServiceError> errors = new ArrayList<>();
        if(!UserValidation.isPrivacySettingsValidToResponseComments(
                userOptional.get().getUserPrivacySettings(),
                errors,
                isLogged) && !Objects.equals(userOptional.get().getEmail(), loggedEmail)){
            return ServiceResult.createServiceResult(errors, HttpStatus.FORBIDDEN);
        }
        List<CommentArticle> commentArticles = commentArticleRepository.findAllByUserId(userId);
        if(commentArticles.isEmpty())
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("user-has-not-comment", "Uživatel nemá žádné komentáře.")),
                    HttpStatus.NOT_FOUND);
        ModelMapper mapper = new ModelMapper();
        List<CommentArticleForUserOut> commentArticleForUserOuts = new ArrayList<>(commentArticles.size());
        commentArticles.forEach(commentArticle -> {
            Optional<Article> articleOptional = articleRepository.findById(commentArticle.getArticle().getId());
            if(articleOptional.isEmpty())
                return;
            CommentArticleForUserOut outTemp = mapper.map(articleOptional.get(), CommentArticleForUserOut.class);
            outTemp.setArticleId(articleOptional.get().getId());
            outTemp.setCommentId(commentArticle.getId());
            outTemp.setComment(commentArticle.getComment());
            outTemp.setCreateTime(commentArticle.getCreateTime());
            outTemp.setIncorrect(commentArticle.getIncorrect());
            commentArticleForUserOuts.add(outTemp);
        });
        return ServiceResult.createServiceResult(commentArticleForUserOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> setCommentIncorrect(Long commentId, Boolean incorrect) {
        Optional<CommentArticle> commentOptional = commentArticleRepository.findById(commentId);
        if(commentOptional.isEmpty())
            return Errors.singletonError("comment-not-found", "Komentář nebyl nalezen", HttpStatus.NOT_FOUND);
        commentOptional.get().setIncorrect(incorrect);
        commentArticleRepository.save(commentOptional.get());
        return ServiceResult.createServiceResult(
                incorrect
                        ? "Změna provedena. Komentář je nevhodný."
                        : "Změna provedena. Komentář je vhodný.",
                HttpStatus.OK);
    }
}
