package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleRatingIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingForUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.RatingArticle;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.JsonFilesRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.RatingArticleRepository;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.validation.UserValidation;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.time.ZonedDateTime;
import java.util.*;

@Service
@RequiredArgsConstructor
public class RatingArticleServiceImpl implements RatingArticleService {

    private final RatingArticleRepository ratingArticleRepository;
    private final UserRepository userRepository;
    private final ArticleRepository articleRepository;

    @Override
    public ServiceResult<String> addRating(ArticleRatingIn articleRatingIn, Integer userID) {
        //check validity
        if (articleRatingIn.getRating() > 5 || articleRatingIn.getRating() < 0) {
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("rating-is-not-valid", "Hodnocení není validní.")), HttpStatus.NOT_ACCEPTABLE);
        }

        Optional<Article> articleOptional = articleRepository.findById(articleRatingIn.getArticleId());
        if (articleOptional.isEmpty()) {
            return Errors.articleNotFound();
        } else if (!articleOptional.get().getVisibility()) {
            return Errors.articleIsNotVisible();
        }

        //check if user havent rate yet
        Optional<RatingArticle> usersRating = ratingArticleRepository.findByUserIdAndArticleId(userID, articleRatingIn.getArticleId());
        if (usersRating.isEmpty()) {
            //save rating
            RatingArticle ratingArticle = new RatingArticle();
            ratingArticle.setArticle(articleOptional.get());
            ratingArticle.setRating(articleRatingIn.getRating());
            ratingArticle.setUser(userRepository.findById(userID).get());
            ratingArticleRepository.save(ratingArticle);
            return ServiceResult.createServiceResult("Hodnocení přidáno.", HttpStatus.OK);
        }
        //update rating
        RatingArticle userRating = usersRating.get();
        userRating.setRating(articleRatingIn.getRating());
        userRating.setCreateTime(ZonedDateTime.now());
        ratingArticleRepository.save(userRating);
        return ServiceResult.createServiceResult("Hodnocení bylo upraveno.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<ArticleRatingOut> getRatingForArticle(Integer articleId) {
        List<RatingArticle> ratingArticlesDB = ratingArticleRepository.findAllByArticleId(articleId);
        if (ratingArticlesDB.isEmpty()) {
            ArticleRatingOut out = new ArticleRatingOut();
            out.setRating(0D);
            out.setCount(0);
            return ServiceResult.createServiceResult(out, HttpStatus.OK);
        }
        ArticleRatingOut articleRatingOut = new ArticleRatingOut();
        if(JsonFilesRepository.generalSettings().getDisplayNumberOfRatings())
            articleRatingOut.setCount(ratingArticlesDB.size());
        articleRatingOut.setRating(ratingArticlesDB.stream().mapToDouble(RatingArticle::getRating).average().getAsDouble());
        return ServiceResult.createServiceResult(articleRatingOut, HttpStatus.OK);
    }

    @Override
    public ServiceResult<ArticleRatingUserOut> getRatingForArticleUser(Integer articleId, Integer userID) {
        Optional<RatingArticle> optionalRatingArticle = ratingArticleRepository.findByUserIdAndArticleId(userID, articleId);
        if (optionalRatingArticle.isEmpty())
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("not-rating-from-user", "Nepřidal jste hodnocení pro tento článek.")), HttpStatus.NOT_FOUND);
        ModelMapper mapper = new ModelMapper();
        return ServiceResult.createServiceResult(mapper.map(optionalRatingArticle.get(), ArticleRatingUserOut.class), HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteRatingForUser(Integer articleId, Integer userID) {
        Optional<RatingArticle> optionalRatingArticle = ratingArticleRepository.findByUserIdAndArticleId(userID, articleId);
        if (optionalRatingArticle.isEmpty())
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("not-rating-from-user", "Nepřidal jste hodnocení pro tento článek.")), HttpStatus.NOT_FOUND);
        ratingArticleRepository.delete(optionalRatingArticle.get());
        return ServiceResult.createServiceResult("Hodnocení bylo odebráno.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<ArticleRatingForUserOut>> getArticleRatingForUser(Integer userId, boolean isLogged, String loggedEmail) {
        Optional<User> userOptional = userRepository.findById(userId);
        if (userOptional.isEmpty())
            return Errors.userNotFound();
        //check privacy
        List<ServiceError> errors = new ArrayList<>();
        if(!UserValidation.isPrivacySettingsValidToResponseRatings(
                userOptional.get().getUserPrivacySettings(),
                errors,
                isLogged) && !Objects.equals(userOptional.get().getEmail(), loggedEmail)){
            return ServiceResult.createServiceResult(errors, HttpStatus.FORBIDDEN);
        }
        List<RatingArticle> ratings = ratingArticleRepository.findAllByUserId(userId);
        if (ratings.isEmpty())
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("user-has-not-ratings", "Uživatel nemá žádná hodnocení.")),
                    HttpStatus.NOT_FOUND);
        ModelMapper mapper = new ModelMapper();
        List<ArticleRatingForUserOut> ratingForUserOuts = new ArrayList<>(ratings.size());
        ratings.forEach(ratingArticle -> {
            Article article = ratingArticle.getArticle();
            ArticleRatingForUserOut articleRatingForUserOut = mapper.map(article, ArticleRatingForUserOut.class);
            articleRatingForUserOut.setRating(ratingArticle.getRating());
            articleRatingForUserOut.setCreateTime(ratingArticle.getCreateTime());
            ratingForUserOuts.add(articleRatingForUserOut);
        });
        return ServiceResult.createServiceResult(ratingForUserOuts, HttpStatus.OK);
    }
}
