package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.constants.FilePaths;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.GalleryImageInfoOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.json.AuthorPageData;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.json.GeneralSettings;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class SettingsServiceImpl implements SettingsService {

    private static final String JSON_REGEX = "(\\\\|\\/).[A-z-]*.json$";

    private final ObjectMapper objectMapper;

    @Override
    public ServiceResult<GeneralSettings> getGeneralSettings() throws IOException {
        if(new File(FilePaths.GENERAL_SETTINGS_PATH).exists()){
            return ServiceResult.createServiceResult(
                    objectMapper.readValue(new File(FilePaths.GENERAL_SETTINGS_PATH), GeneralSettings.class),
                    HttpStatus.OK);
        }
        return ServiceResult.createServiceResult(new GeneralSettings(), HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> updateGeneralSettings(GeneralSettings generalSettings) {
        try {
            if(!new File(FilePaths.GENERAL_SETTINGS_PATH).exists()){
                Files.createDirectories(
                        Paths.get(FilePaths.GENERAL_SETTINGS_PATH.replaceFirst(JSON_REGEX,"")));
            }
            objectMapper.writeValue(new File(FilePaths.GENERAL_SETTINGS_PATH), generalSettings);
            return ServiceResult.createServiceResult("Nastavení uloženo", HttpStatus.OK);
        } catch (IOException e) {
            e.printStackTrace();
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError("saving-settings-failed", "Chyba při ukládání nastavení"),
                            new ServiceError("ioexception", e.getMessage())),
                    HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public ServiceResult<AuthorPageData> getAuthorPageData() throws IOException {
        if(new File(FilePaths.ABOUT_AUTHOR_DATA_PATH).exists()){
            return ServiceResult.createServiceResult(
                    objectMapper.readValue(new File(FilePaths.ABOUT_AUTHOR_DATA_PATH), AuthorPageData.class),
                    HttpStatus.OK);
        }
        return ServiceResult.createServiceResult(new AuthorPageData(), HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> updateAuthorPageData(AuthorPageData authorPageData) {
        try{
            if(!new File(FilePaths.ABOUT_AUTHOR_DATA_PATH).exists()){
                Files.createDirectories(
                        Paths.get(FilePaths.ABOUT_AUTHOR_DATA_PATH.replaceFirst(JSON_REGEX,"")));
            }
            objectMapper.writeValue(new File(FilePaths.ABOUT_AUTHOR_DATA_PATH), authorPageData);
            return ServiceResult.createServiceResult("Nastavení uloženo", HttpStatus.OK);
        } catch (IOException e) {
            e.printStackTrace();
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError("saving-settings-failed", "Chyba při ukládání nastavení"),
                            new ServiceError("ioexception", e.getMessage())),
                    HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public byte[] getFavicon() throws IOException {
        if(!new File(FilePaths.FAVICON_PATH).exists()) {
            InputStream in = getClass()
                    .getResourceAsStream("/static/favicon.ico");
            assert in != null;
            return in.readAllBytes();
        } else {
            return Files.readAllBytes(Path.of(FilePaths.FAVICON_PATH));
        }
    }

    @Override
    public ServiceResult<String> updateFavicon(byte[] icon) throws IOException {
        if(!new File(FilePaths.FAVICON_PATH).exists()){
            Files.createDirectories(
                    Paths.get(FilePaths.FAVICON_PATH.replaceFirst("/favicon.ico","")));
        }
        Files.write(Path.of(FilePaths.FAVICON_PATH), icon);
        return ServiceResult.createServiceResult("Ikona uložena", HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> uploadImages(MultipartFile[] files) throws IOException {
        if(!new File(FilePaths.GALLERY_FOLDER).exists()){
            Files.createDirectories(Paths.get(FilePaths.GALLERY_FOLDER));
        }
        Arrays.stream(files).forEach(multipartFile -> {
            try {
                String extension = multipartFile.getOriginalFilename().substring(multipartFile.getOriginalFilename().lastIndexOf(".")+1);
                multipartFile.transferTo(Path.of(createNewImagePath(extension)));
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        return ServiceResult.createServiceResult("Hotovo", HttpStatus.OK);
    }

    @Override
    public ResponseEntity<byte[]> getImage(String imageName) throws IOException {
        if(new File(FilePaths.GALLERY_FOLDER + imageName).exists()){
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.valueOf(Files.probeContentType(Path.of(FilePaths.GALLERY_FOLDER + imageName))));
            return new ResponseEntity<>(
                    Files.readAllBytes(Path.of(FilePaths.GALLERY_FOLDER + imageName)),
                    headers,
                    HttpStatus.OK);
        }
        return new ResponseEntity<>(null, HttpStatus.NO_CONTENT);
    }

    @Override
    public ServiceResult<List<GalleryImageInfoOut>> getAllImagesNames() {
        if(!new File(FilePaths.GALLERY_FOLDER).exists()){
            return Errors.singletonError("empty-gallery", "Galerie neobsahuje žádné obrázky.", HttpStatus.NOT_FOUND);
        }
        List<GalleryImageInfoOut> out = new ArrayList<>();
        Arrays.stream(Objects.requireNonNull(new File(FilePaths.GALLERY_FOLDER).list())).forEach(name ->{
            try {
                GalleryImageInfoOut temp = new GalleryImageInfoOut();
                temp.setName(name);
                BufferedImage imageTemp = ImageIO.read(new File(FilePaths.GALLERY_FOLDER + name));
                temp.setWidth(imageTemp.getWidth());
                temp.setHeight(imageTemp.getHeight());
                out.add(temp);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        if(out.isEmpty())
            return Errors.singletonError("empty-gallery", "Galerie neobsahuje žádné obrázky.", HttpStatus.NOT_FOUND);
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteImage(String imageName) {
        if(!new File(FilePaths.GALLERY_FOLDER + imageName).exists())
            return Errors.singletonError("image-not-found", "Obrázek nebyl nalezen.", HttpStatus.NOT_FOUND);
        File file = new File(FilePaths.GALLERY_FOLDER + imageName);
        if(!file.delete())
            return Errors.singletonError("image-delete-failure", "Chyba při mazání obrázku", HttpStatus.INTERNAL_SERVER_ERROR);
        return ServiceResult.createServiceResult("Obrázek byl odstraněn", HttpStatus.OK);
    }

    private String createNewImagePath(String extension){
        int sumOfImages = Objects.requireNonNull(new File(FilePaths.GALLERY_FOLDER).list()).length;
        StringBuilder name = new StringBuilder("image_");
        name.append("0".repeat(Math.max(0, 5 - String.valueOf(sumOfImages).length())));
        name.append(sumOfImages);
        return FilePaths.GALLERY_FOLDER + name + "." + extension;
    }
}
