package cz.uhk.fim.redakcnisystem.usermanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingForUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.CommentArticleForUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.CommentArticleService;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.RatingArticleService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserAdditionalInfoIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserDeletePasswordIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserPasswordChangeIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserPrivacySettingsIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.*;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserImage;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserSavedArticleService;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

@RequiredArgsConstructor
@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping("api/v1/user-management")
public class UserController {

    private final UserService userService;
    private final UserSavedArticleService userSavedArticleService;
    private final CommentArticleService commentArticleService;
    private final RatingArticleService ratingArticleService;


    @Secured("ROLE_USER")
    @GetMapping("/current-user")
    public ResponseEntity<ServerResponse<CurrentUserOut>> currentUser(@RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(
                userService.currentUser(userId));
    }

    @GetMapping("/user/{id}")
    public ResponseEntity<ServerResponse<UserOut>> getUser(
            @PathVariable("id") String userId,
            @RequestHeader(value = HttpHeaders.AUTHORIZATION, required = false) String tokenHeader){
        boolean isLogged = JwtUtil.validateTokenFromHeader(tokenHeader);
        String loggedEmail = isLogged ? JwtUtil.extractUsernameFromHeader(tokenHeader) : null;
        return ServerResponse.createResponseEntity(userService.getUser(userId, isLogged, loggedEmail));
    }

    @Secured("ROLE_USER")
    @PostMapping("/user/article/{articleId}/saved")
    public ResponseEntity<ServerResponse<String>> userSaveArticle(
            @PathVariable Integer articleId,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userSavedArticleService.saveUserArticle(userId, articleId));
    }

    @Secured("ROLE_USER")
    @GetMapping("/user/article/{articleId}/saved")
    public ResponseEntity<ServerResponse<UserSavedArticleOut>> getUserSavedArticleForArticle(
            @PathVariable Integer articleId,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userSavedArticleService.getUserSavedArticleForArticle(userId, articleId));
    }

    @Secured("ROLE_USER")
    @DeleteMapping("/user/article/saved/{id}")
    public ResponseEntity<ServerResponse<String>> deleteUserSavedArticle(
            @PathVariable Long id,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userSavedArticleService.deleteUserSavedArticle(id, userId));
    }

    @Secured("ROLE_USER")
    @GetMapping("/user/article/saved")
    public ResponseEntity<ServerResponse<List<UserSavedArticleForListOut>>> getListOfSavedArticles(
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userSavedArticleService.getListOfSavedArticles(userId));
    }

    //change password
    @Secured("ROLE_USER")
    @PutMapping("/user/password")
    public ResponseEntity<ServerResponse<String>> editUserPassword(
            @RequestBody UserPasswordChangeIn userPasswordChangeIn,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userService.editUserPassword(userPasswordChangeIn, userId));
    }

    //change user additional information
    @Secured("ROLE_USER")
    @PutMapping("/user")
    public ResponseEntity<ServerResponse<String>> editUserAdditionalInformation(
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader,
            @RequestBody UserAdditionalInfoIn userAdditionalInfoIn){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userService.editUserAdditionalInformation(userId, userAdditionalInfoIn));
    }

    //delete user
    @Secured("ROLE_USER")
    @DeleteMapping("/user")
    public ResponseEntity<ServerResponse<String>> deleteUser(
            @RequestBody UserDeletePasswordIn userDeletePasswordIn,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userService.deleteUser(userId, userDeletePasswordIn));
    }

    @Secured("ROLE_USER")
    @GetMapping("/user/privacy-settings")
    public ResponseEntity<ServerResponse<UserPrivacySettingsOut>> getUserPrivacySettings(
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userService.getUserPrivacySettings(userId));
    }

    @Secured("ROLE_USER")
    @PutMapping("/user/privacy-settings")
    public ResponseEntity<ServerResponse<String>> editUserPrivacySettings(
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader,
            @RequestBody UserPrivacySettingsIn userPrivacySettingsIn){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(userService.editUserPrivacySettings(userId, userPrivacySettingsIn));
    }

    @Secured("ROLE_USER")
    @PostMapping("/user/image")
    public ResponseEntity<ServerResponse<String>> postUserImage(
            @RequestParam("image") MultipartFile image,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader) throws IOException {
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(
                userService.postImageUser(image, userId));
    }

    @GetMapping("/user/{id}/image")
    public ResponseEntity<?> getUserImage(@PathVariable String id){
        ServiceResult<UserImage> userImageServiceResult = userService.getUserImage(id);
        if(userImageServiceResult.getHttpStatus() != HttpStatus.OK){
            return ServerResponse.createResponseEntity(userImageServiceResult);
        }
        return ResponseEntity
                .status(HttpStatus.OK)
                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + userImageServiceResult.getBody().getName() + ".png\"")
                .contentType(MediaType.valueOf("image/png"))
                .body(userImageServiceResult.getBody().getImageData());
    }

    @GetMapping("/user/{id}/comment")
    public ResponseEntity<ServerResponse<List<CommentArticleForUserOut>>> getArticleCommentForUser(
            @PathVariable String id,
            @RequestHeader(value = HttpHeaders.AUTHORIZATION, required = false) String tokenHeader){
        boolean isLogged = JwtUtil.validateTokenFromHeader(tokenHeader);
        String loggedEmail = isLogged ? JwtUtil.extractUsernameFromHeader(tokenHeader) : null;
        return ServerResponse.createResponseEntity(
                commentArticleService.getArticleCommentsForUser(Integer.valueOf(id), isLogged, loggedEmail));
    }

    @GetMapping("/user/{id}/rating")
    public ResponseEntity<ServerResponse<List<ArticleRatingForUserOut>>> getRatingForUser(
            @PathVariable String id,
            @RequestHeader(value = HttpHeaders.AUTHORIZATION, required = false) String tokenHeader){
        boolean isLogged = JwtUtil.validateTokenFromHeader(tokenHeader);
        String loggedUsername = isLogged ? JwtUtil.extractUsernameFromHeader(tokenHeader) : null;
        return ServerResponse.createResponseEntity(
                ratingArticleService.getArticleRatingForUser(Integer.valueOf(id), isLogged, loggedUsername));
    }

}
