package cz.uhk.fim.redakcnisystem.usermanagement.service;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserLoginIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserRegistrationIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserTokenOut;
import cz.uhk.fim.redakcnisystem.usermanagement.records.*;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.UserRoleEn;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.*;
import cz.uhk.fim.redakcnisystem.usermanagement.validation.UserValidation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.codec.digest.DigestUtils;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class AuthServiceImpl implements AuthService{

    private final UserRepository userRepository;
    private final UserImageRepository userImageRepository;
    private final RoleRepository roleRepository;
    private final UserService userService;
    private final UserPrivacySettingsRepository privacySettingsRepository;
    private final UserLoginActivityRepository userLoginActivityRepository;

    private final ModelMapper mapper = new ModelMapper();

    @Override
    public ServiceResult<String> registration(UserRegistrationIn userRegistrationIn, MultipartFile image) {
        List<ServiceError> errors = new ArrayList<>();
        //validace
        UserValidation.validateRegistration(errors, userRegistrationIn);
        // kontrola zda se hesla použité při registraci shodují
        if(!userRegistrationIn.getPassword().equals(userRegistrationIn.getConfirmPassword())){
            errors.add(new ServiceError("passwords-not-match","Hesla se neshodují."));
        }
        if(!errors.isEmpty()){
            return ServiceResult.createServiceResult(errors, HttpStatus.NOT_ACCEPTABLE);
        }
        // kontrola zda existuje uživatel podle emailu
        Optional<User> optionalUser = userRepository.findByEmail(userRegistrationIn.getEmail());
        if(optionalUser.isPresent()){
            return ServiceResult.createServiceResult(
                    Collections.singletonList(new ServiceError("email-exists","Uživatel s tímto emailem, je již registrován.")),
                    HttpStatus.NOT_ACCEPTABLE);
        }
        // kontrola zda existuje uživatel podle jména
        optionalUser = userRepository.findByUsername(userRegistrationIn.getUsername());
        if(optionalUser.isPresent()){
            return ServiceResult.createServiceResult(
                    Collections.singletonList(new ServiceError("username-exists","Uživatel s tímto jménem, je již registrován.")),
                    HttpStatus.NOT_ACCEPTABLE);
        }
        userRegistrationIn.setPassword(DigestUtils.sha256Hex(userRegistrationIn.getPassword()));
        User userEntity = mapper.map(userRegistrationIn, User.class);
        Long countAll = userRepository.count();
        userRepository.save(userEntity);
        //default role
        UserRole userRole = new UserRole();
        if(countAll == 0) {
            UserRole firstUserRole = new UserRole();
            firstUserRole.setRole(UserRoleEn.ADMIN);
            firstUserRole.setUser(userEntity);
            roleRepository.save(firstUserRole);
        }
        userRole.setRole(UserRoleEn.USER);
        userRole.setUser(userEntity);
        roleRepository.save(userRole);
        // default privacy
        UserPrivacySettings userPrivacySettings = new UserPrivacySettings();
        userPrivacySettings.setUser(userEntity);
        privacySettingsRepository.save(userPrivacySettings);
        // image for user
        errors.clear();
        try{
            boolean useDefault = image == null;
            if(!useDefault){
                ServiceResult<String> imageServiceResult = userService.postImageUser(image, userEntity.getId());
                if(imageServiceResult.getErrors() != null){
                    useDefault = true;
                    errors.addAll(imageServiceResult.getErrors());
                    errors.add(new ServiceError("default-image-used", "Byl použit výchozí obrázek, lze změnit v nastavení profilu."));
                }
            }
            if(useDefault){
                InputStream inputStream = getClass().getClassLoader().getResourceAsStream("images/default-profile-image.png");
                MultipartFile defaultImage = new MockMultipartFile("default.png", inputStream);
                UserImage userImage = new UserImage();
                userImage.setUser(userEntity);
                userImage.setName(defaultImage.getName());
                userImage.setType("image/png");
                userImage.setImageData(defaultImage.getBytes());
                userImageRepository.save(userImage);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        ServiceResult<String> result = ServiceResult.createServiceResult("Uživatel byl vytvořen.",HttpStatus.OK);
        if(errors.isEmpty())
            return result;
        result.setErrors(errors);
        result.setHttpStatus(HttpStatus.MULTI_STATUS);
        return result;
    }

    @Override
    public ServiceResult<UserTokenOut> login(UserLoginIn userLoginIn) {
        List<ServiceError> errors = new ArrayList<>();
        // validace vstupních hodnot
        UserValidation.validateLogin(errors, userLoginIn);
        if(!errors.isEmpty()){
            return ServiceResult.createServiceResult(errors, HttpStatus.NOT_ACCEPTABLE);
        }
        //šifrování hesla
        userLoginIn.setPassword(DigestUtils.sha256Hex(userLoginIn.getPassword()));
        Optional<User> userOptional = userRepository.findByEmail(userLoginIn.getEmail());
        // kontrola zda uživatel podle emailu existuje
        if(userOptional.isEmpty()){
            return Errors.userNotFound();
        }
        // kontrola zda se shodují hesla
        if(!userOptional.get().getPassword().equals(userLoginIn.getPassword())){
            return ServiceResult.createServiceResult(
                    Collections.singletonList(new ServiceError("wrong-password","Nesprávné heslo."))
                    ,HttpStatus.NOT_ACCEPTABLE);
        }
        // návrat uživatele
        //UserOut userOut = mapper.map(userOptional.get(), UserOut.class);
        UserTokenOut userTokenOut = UserTokenOut.builder()
                .token(JwtUtil.generateToken(userOptional.get().getEmail()))
                .username(userOptional.get().getUsername())
                .id(userOptional.get().getId())
                .build();
        userTokenOut.setRolesFromUserRoles(userOptional.get().getRoles());
        updateLastLogin(userOptional.get().getId());
        return ServiceResult.createServiceResult(userTokenOut, HttpStatus.OK);
    }

    private void updateLastLogin(Integer userId) {
        Optional<User> optionalUser = userRepository.findById(userId);
        if(optionalUser.isPresent()){
            UserLoginActivity userLoginActivity = new UserLoginActivity();
            userLoginActivity.setUser(optionalUser.get());
            userLoginActivityRepository.save(userLoginActivity);
        }
    }
}

