package cz.uhk.fim.redakcnisystem.usermanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleRepository;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserSavedArticleForListOut;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserSavedArticleOut;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserSavedArticle;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserSavedArticleRepository;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class UserSavedArticleServiceImpl implements UserSavedArticleService{

    private final UserRepository userRepository;
    private final UserSavedArticleRepository userSavedArticleRepository;
    private final ArticleRepository articleRepository;

    @Override
    public ServiceResult<String> saveUserArticle(Integer userId, Integer articleId) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        if(articleOptional.isEmpty())
            return Errors.articleNotFound();
        if(!articleOptional.get().getVisibility())
            return Errors.articleIsNotVisible();
        Optional<UserSavedArticle> optionalUserSavedArticle = userSavedArticleRepository.findByArticleIdAndUserId(articleId, userId);
        if(optionalUserSavedArticle.isPresent())
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError(
                                    "article-is-already-saved-by-user",
                                    "Uživatel již má uložený tento článek.")),
                    HttpStatus.FORBIDDEN);
        UserSavedArticle userSavedArticle = new UserSavedArticle();
        userSavedArticle.setArticle(articleOptional.get());
        userSavedArticle.setUser(userOptional.get());
        userSavedArticleRepository.save(userSavedArticle);
        return ServiceResult.createServiceResult("Článek uložen do seznamu.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<UserSavedArticleOut> getUserSavedArticleForArticle(Integer userId, Integer articleId) {
        Optional<UserSavedArticle> optionalUserSavedArticle = userSavedArticleRepository.findByArticleIdAndUserId(articleId, userId);
        if(optionalUserSavedArticle.isEmpty())
            return ServiceResult.createServiceResult(List.of(
                    new ServiceError(
                            "article-is-not-saved-by-user",
                            "Uživatel nemá uložený tento článek")),
                    HttpStatus.NOT_FOUND);
        ModelMapper mapper = new ModelMapper();
        UserSavedArticleOut userSavedArticleOut = mapper.map(optionalUserSavedArticle.get(), UserSavedArticleOut.class);
        return ServiceResult.createServiceResult(userSavedArticleOut, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteUserSavedArticle(Long id, Integer userId) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        Optional<UserSavedArticle> userSavedArticleOptional = userSavedArticleRepository.findById(id);
        if(userSavedArticleOptional.isEmpty())
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError(
                                    "article-is-not-saved-by-user",
                                    "Uživatel nemá uložený tento článek")),
                    HttpStatus.NOT_FOUND);
        if(!userSavedArticleOptional.get().getUser().getId().equals(userOptional.get().getId()))
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError(
                                    "user-is-not-owner-of-article-saved-by-user",
                                    "Uživatel nemůže odebrat toto uložení.")),
                    HttpStatus.NOT_FOUND);
        userSavedArticleRepository.delete(userSavedArticleOptional.get());
        return ServiceResult.createServiceResult("Článek byl odebrán ze seznamu.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<UserSavedArticleForListOut>> getListOfSavedArticles(Integer userId) {
        List<UserSavedArticle> userSavedArticles = userSavedArticleRepository.findAllByUserId(userId);
        if(userSavedArticles.isEmpty())
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError(
                                    "user-article-saved-list-empty",
                                    "Uživatel nemá uložený žádný článek.")),
                    HttpStatus.NOT_FOUND);
        List<UserSavedArticleForListOut> forListOuts = new ArrayList<>();
        userSavedArticles.forEach(userSavedArticle -> {
            UserSavedArticleForListOut outTemp = new UserSavedArticleForListOut();
            outTemp.setId(userSavedArticle.getId());
            outTemp.setArticleId(userSavedArticle.getArticle().getId());
            outTemp.setTitle(userSavedArticle.getArticle().getTitle());
            forListOuts.add(outTemp);
        });
        return ServiceResult.createServiceResult(forListOuts, HttpStatus.OK);
    }
}
