package cz.uhk.fim.redakcnisystem.usermanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleViewActivityRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.CommentArticleRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.RatingArticleRepository;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserAdditionalInfoIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserDeletePasswordIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserPasswordChangeIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserPrivacySettingsIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.CurrentUserOut;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserAdminOut;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserOut;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserPrivacySettingsOut;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserImage;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserLoginActivity;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserPrivacySettings;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserImageRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserLoginActivityRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserPrivacySettingsRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.validation.UserValidation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.codec.digest.DigestUtils;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

@Service
@RequiredArgsConstructor
public class UserServiceImpl implements UserService{

    private final UserRepository userRepository;
    private final UserImageRepository userImageRepository;
    private final RatingArticleRepository ratingArticleRepository;
    private final CommentArticleRepository commentArticleRepository;
    private final UserPrivacySettingsRepository userPrivacySettingsRepository;
    private final UserLoginActivityRepository userLoginActivityRepository;
    private final ArticleViewActivityRepository articleViewActivityRepository;

    private final ModelMapper mapper = new ModelMapper();

    @Override
    public boolean userIsValid(Integer userID) {
        Optional<User> userOptional = userRepository.findById(userID);
        return userOptional.isPresent();
    }

    @Override
    public ServiceResult<CurrentUserOut> currentUser(Integer userID) {
        Optional<User> userOptional = userRepository.findById(userID);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        CurrentUserOut currentUserOut = mapper.map(userOptional.get(), CurrentUserOut.class);
        currentUserOut.setRolesFromUserRoles(userOptional.get().getRoles());
        return ServiceResult.createServiceResult(currentUserOut, HttpStatus.OK);
    }

    @Override
    public ServiceResult<UserOut> getUser(String userId, boolean isLogged, String loggedEmail) {
        Optional<User> userOptional = userRepository.findById(Integer.valueOf(userId));
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        UserOut userOut = mapper.map(userOptional.get(), UserOut.class);
        userOut.setRegistrationTime(userOptional.get().getCreateTime());
        //skip privacy check, logged user is the requested one
        if(isLogged && Objects.equals(loggedEmail, userOptional.get().getEmail()))
            return ServiceResult.createServiceResult(userOut, HttpStatus.OK);
        //privacy check
        List<ServiceError> errors = new ArrayList<>();
        UserValidation.userOutPrivacyValidation(userOut, userOptional.get().getUserPrivacySettings(), isLogged, errors);
        if(errors.isEmpty())
            return ServiceResult.createServiceResult(userOut, HttpStatus.OK);
        ServiceResult<UserOut> result = ServiceResult.createServiceResult(userOut, HttpStatus.PARTIAL_CONTENT);
        result.setErrors(errors);
        return result;
    }

    @Override
    public ServiceResult<String> editUserPassword(UserPasswordChangeIn userPasswordChangeIn, Integer userId) {
        //todo předělat kontrolu na validace viz UserValidation
        if(userPasswordChangeIn.getOldPassword() != null && !userPasswordChangeIn.getOldPassword().equals("")){ // změna hesla
           if(userPasswordChangeIn.getConfirmPassword().equals(userPasswordChangeIn.getPassword())){ // kontrola rovnosti nových hesel
               Optional<User> userOptional = userRepository.findById(userId);
               if(userOptional.isEmpty())
                   return Errors.userNotFound();
               User userEntity = userOptional.get();
               if(!userEntity.getPassword().equals(DigestUtils.sha256Hex(userPasswordChangeIn.getOldPassword())))
                   return ServiceResult.createServiceResult(
                           Collections.singletonList(new ServiceError("wrong-current-password", "Stávající heslo je chybné.")), HttpStatus.FORBIDDEN);
               userEntity.setPassword(DigestUtils.sha256Hex(userPasswordChangeIn.getPassword()));
               return ServiceResult.createServiceResult("Heslo bylo změněno.", HttpStatus.OK);
           } else {
               return ServiceResult.createServiceResult(
                       Collections.singletonList(new ServiceError("passwords-not-match", "Hesla se neshodují.")), HttpStatus.FORBIDDEN);
           }
        }
        return ServiceResult.createServiceResult("Nedošlo k žádné změně.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> postImageUser(MultipartFile image, Integer userID) throws IOException {
        String[] extensions = {".jpg",".jpeg",".png"};
        if(Arrays.stream(extensions).noneMatch(ext -> Objects.requireNonNull(image.getResource().getFilename()).toLowerCase().endsWith(ext))){
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("incorrect-image-format", "Špatný formát obrázku.")),
                    HttpStatus.NOT_ACCEPTABLE);
        }
        Optional<User> userOptional = userRepository.findById(userID);
        if(userOptional.isEmpty()){
            return Errors.userNotFound();
        }

        Optional<UserImage> optionalUserImage = userImageRepository.findByUserId(userOptional.get().getId());
        optionalUserImage.ifPresent(userImageRepository::delete);

        UserImage userImage = new UserImage();
        userImage.setUser(userOptional.get());
        userImage.setName(image.getOriginalFilename());
        userImage.setType(image.getContentType());
        userImage.setImageData(image.getBytes());
        userImageRepository.save(userImage);
        return ServiceResult.createServiceResult("Obrázek byl nahrán.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<UserImage> getUserImage(String userId){
        Optional<User> userOptional = userRepository.findById(Integer.valueOf(userId));
        if(userOptional.isEmpty()) {
            return Errors.userNotFound();
        }
        Optional<UserImage> userImageOptional = userImageRepository.findByUserId(Integer.valueOf(userId));
        if(userImageOptional.isEmpty()) {
            return ServiceResult.createServiceResult(
                    Collections.singletonList(
                            new ServiceError("user-image-not-found", "Obrázek uživatele nebyl nalezen.")),
                    HttpStatus.NOT_FOUND);
        }
        return ServiceResult.createServiceResult(userImageOptional.get(), HttpStatus.OK);
    }

    @Override
    public Integer getUserId(String email) {
        return userRepository.findByEmail(email).get().getId();
    }

    @Override
    public ServiceResult<List<UserAdminOut>> getUsersListAdmin() {
        List<UserAdminOut> userAdminOuts = new ArrayList<>();
        userRepository.findAll().forEach(user -> {
            UserAdminOut userAdmin = mapper.map(user, UserAdminOut.class);
            userAdmin.setRatingsCount(ratingArticleRepository.countAllByUserId(user.getId()));
            userAdmin.setCommentsCount(commentArticleRepository.countAllByUserId(user.getId()));
            userAdmin.setRoles(new ArrayList<>());
            Optional<UserLoginActivity> userLoginActivity = userLoginActivityRepository.findFirstByUserOrderByLoginTimeDesc(user);
            userLoginActivity.ifPresent(loginActivity -> userAdmin.setLastLoginTime(loginActivity.getLoginTime()));
            user.getRoles().forEach(userRole -> userAdmin.getRoles().add(userRole.getRole()));
            userAdminOuts.add(userAdmin);
        });
        if(!userAdminOuts.isEmpty())
            return ServiceResult.createServiceResult(userAdminOuts, HttpStatus.OK);
        return null;
    }

    @Override
    public ServiceResult<UserPrivacySettingsOut> getUserPrivacySettings(Integer userId) {
        Optional<UserPrivacySettings> privacySettingsOptional = userPrivacySettingsRepository.findByUserId(userId);
        if(privacySettingsOptional.isEmpty())
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError(
                                    "user-privacy-settings-not-found",
                                    "Nastavení soukromí nebylo nalezeno.")),
                    HttpStatus.NOT_FOUND);
        return ServiceResult.createServiceResult(mapper.map(privacySettingsOptional.get(), UserPrivacySettingsOut.class), HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> editUserPrivacySettings(Integer userId, UserPrivacySettingsIn privacySettingsIn) {
        Optional<UserPrivacySettings> privacySettingsOptional = userPrivacySettingsRepository.findByUserId(userId);
        if(privacySettingsOptional.isEmpty())
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError(
                                    "user-privacy-settings-not-found",
                                    "Nastavení soukromí nebylo nalezeno.")),
                    HttpStatus.NOT_FOUND);
        UserPrivacySettings userPrivacySettings = privacySettingsOptional.get();
        userPrivacySettings.setEmailPrivacy(privacySettingsIn.getEmailPrivacy());
        userPrivacySettings.setRegistrationTimePrivacy(privacySettingsIn.getRegistrationTimePrivacy());
        userPrivacySettings.setRatingsPrivacy(privacySettingsIn.getRatingsPrivacy());
        userPrivacySettings.setCommentsPrivacy(privacySettingsIn.getCommentsPrivacy());
        userPrivacySettings.setBioPrivacy(privacySettingsIn.getBioPrivacy());
        userPrivacySettings.setBirthDatePrivacy(privacySettingsIn.getBirthDatePrivacy());
        userPrivacySettings.setGenderPrivacy(privacySettingsIn.getGenderPrivacy());
        userPrivacySettingsRepository.save(userPrivacySettings);
        return ServiceResult.createServiceResult("Změna provedena.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> deleteUser(Integer userId, UserDeletePasswordIn userDeletePasswordIn) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError("user-not-exists", "Uživatel nebyl nalezen.")),
                    HttpStatus.NO_CONTENT);

        if(!Objects.equals(userOptional.get().getPassword(), DigestUtils.sha256Hex(userDeletePasswordIn.getPassword())))
            return ServiceResult.createServiceResult(List.of(
                    new ServiceError("wrong-password", "Špatné heslo.")),
                    HttpStatus.NOT_ACCEPTABLE);

        articleViewActivityRepository.findAllByUserId(userId).forEach(view ->{
            view.setUser(null);
            articleViewActivityRepository.save(view);
        });
        userRepository.delete(userOptional.get());
        return ServiceResult.createServiceResult("Účet byl odebrán, budete přesunut na hlavní stránku.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> editUserAdditionalInformation(Integer userId, UserAdditionalInfoIn userAdditionalInfoIn) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return ServiceResult.createServiceResult(List.of(
                            new ServiceError("user-not-exists", "Uživatel nebyl nalezen.")),
                    HttpStatus.NO_CONTENT);
        User user = userOptional.get();
        user.setBirthDate(userAdditionalInfoIn.getBirthDate());
        user.setBio(userAdditionalInfoIn.getBio());
        user.setGender(userAdditionalInfoIn.getGender());
        userRepository.save(user);
        return ServiceResult.createServiceResult("Změna provedena.", HttpStatus.OK);
    }
}
