package cz.uhk.fim.redakcnisystem.usermanagement.validation;

import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserLoginIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserRegistrationIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserOut;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserPrivacySettings;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.PrivacyTypeEn;
import org.modelmapper.ModelMapper;

import java.util.List;

public class UserValidation {
    private static final String REG_EX_USERNAME = "^[A-ZÁ-Ža-zá-ž\\d ]{1,32}$";
    private static final String REG_EX_EMAIL = "^[\\w\\.\\-]{1,50}@[a-z\\d]{1,10}\\.[a-z\\s]{2,10}$";
    private static final String REG_EX_PASSWORD = "^(?=.*\\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[a-zA-Z]).{4,}$";

    private static final String privacyPrivateContent = "Tento údaj je soukromý.";
    private static final String privacyLoggedContent = "Tento údaj je viditelný po příhlášení.";

    public static void userOutPrivacyValidation(UserOut user, UserPrivacySettings privacySettings, boolean isLogged, List<ServiceError> errors){
        switch (privacySettings.getEmailPrivacy()){
            case PRIVATE -> {
                user.setEmail(null);
                errors.add(privacyError("email", PrivacyTypeEn.PRIVATE));
            }
            case LOGGED -> {
                if(!isLogged) {
                    user.setEmail(null);
                    errors.add(privacyError("email", PrivacyTypeEn.LOGGED));
                }
            }
        }
        switch (privacySettings.getRegistrationTimePrivacy()){
            case PRIVATE -> {
                user.setRegistrationTime(null);
                errors.add(privacyError("registration-time", PrivacyTypeEn.PRIVATE));
            }
            case LOGGED -> {
                if(!isLogged) {
                    user.setRegistrationTime(null);
                    errors.add(privacyError("registration-time", PrivacyTypeEn.LOGGED));
                }
            }
        }
        switch (privacySettings.getBioPrivacy()){
            case PRIVATE -> {
                user.setBio(null);
                errors.add(privacyError("bio", PrivacyTypeEn.PRIVATE));
            }
            case LOGGED -> {
                if(!isLogged) {
                    user.setBio(null);
                    errors.add(privacyError("bio", PrivacyTypeEn.LOGGED));
                }
            }
        }
        switch (privacySettings.getBirthDatePrivacy()){
            case PRIVATE -> {
                user.setBirthDate(null);
                errors.add(privacyError("birth-date", PrivacyTypeEn.PRIVATE));
            }
            case LOGGED -> {
                if(!isLogged) {
                    user.setBirthDate(null);
                    errors.add(privacyError("birth-date", PrivacyTypeEn.LOGGED));
                }
            }
        }
        switch (privacySettings.getGenderPrivacy()){
            case PRIVATE -> {
                user.setGender(null);
                errors.add(privacyError("gender", PrivacyTypeEn.PRIVATE));
            }
            case LOGGED -> {
                if(!isLogged) {
                    user.setGender(null);
                    errors.add(privacyError("gender", PrivacyTypeEn.LOGGED));
                }
            }
        }
    }

    public static boolean isPrivacySettingsValidToResponseComments(
            UserPrivacySettings privacySettings,
            List<ServiceError> errors,
            boolean isLogged){
        switch (privacySettings.getCommentsPrivacy()){
            case LOGGED -> {
                if(isLogged) return true;
                errors.add(privacyError("comments", PrivacyTypeEn.LOGGED));
                return false;
            }
            case PRIVATE -> {
                errors.add(privacyError("comments", PrivacyTypeEn.PRIVATE));
                return false;
            }
        }
        return true;
    }

    public static boolean isPrivacySettingsValidToResponseRatings(
            UserPrivacySettings privacySettings,
            List<ServiceError> errors,
            boolean isLogged){
        switch (privacySettings.getRatingsPrivacy()){
            case LOGGED -> {
                if(isLogged) return true;
                errors.add(privacyError("ratings", PrivacyTypeEn.LOGGED));
                return false;
            }
            case PRIVATE -> {
                errors.add(privacyError("ratings", PrivacyTypeEn.PRIVATE));
                return false;
            }
        }
        return true;
    }

    private static ServiceError privacyError(String parameterName, PrivacyTypeEn privacyType){
        return new ServiceError(
                "privacy-"+parameterName,
                (privacyType == PrivacyTypeEn.LOGGED)
                        ? privacyLoggedContent
                        : privacyPrivateContent);
    }

    public static void validateRegistration(List<ServiceError> errors, UserRegistrationIn registrationIn) {
        ModelMapper mapper = new ModelMapper();
        validateLogin(errors, mapper.map(registrationIn, UserLoginIn.class));
        if(registrationIn.getUsername() == null || registrationIn.getUsername().replaceAll("\\s", "").isEmpty()){
            errors.add(new ServiceError("username-not-specified", "Nebylo zadáno uživatelské jméno."));
        } else if(!registrationIn.getUsername().matches(REG_EX_USERNAME)){
            errors.add(new ServiceError("username-invalid-syntax", "Uživatelské jméno bylo špatně zadáno."));
        }
        if(registrationIn.getConfirmPassword() == null || registrationIn.getConfirmPassword().replaceAll("\\s", "").isEmpty()){
            errors.add(new ServiceError("password-confirm-not-specified", "Nebylo zadáno potvruzíjící heslo."));
        } else if(!registrationIn.getConfirmPassword().matches(REG_EX_PASSWORD)){
            errors.add(new ServiceError("password-confirm-invalid-syntax", "Potvrzení hesla bylo špatně zadáno."));
        }
    }

    public static void validateLogin(List<ServiceError> errors, UserLoginIn loginIn){
        if(loginIn.getEmail() == null || loginIn.getEmail().replaceAll("\\s", "").isEmpty()){
            errors.add(new ServiceError("email-not-specified", "Nebyl zadán email."));
        } else if(!loginIn.getEmail().matches(REG_EX_EMAIL)){
            errors.add(new ServiceError("email-invalid-syntax", "Email byl špatně zadán."));
        }
        loginIn.setEmail(loginIn.getEmail().replaceAll("\\s",""));
        if(loginIn.getPassword() == null || loginIn.getPassword().replaceAll("\\s", "").isEmpty()){
            errors.add(new ServiceError("password-not-specified", "Nebylo zadáno heslo."));
        } else if(!loginIn.getPassword().matches(REG_EX_PASSWORD)){
            errors.add(new ServiceError("password-invalid-syntax", "Heslo bylo špatně zadáno."));
        }
    }
}
