import React, {useContext, useEffect, useState} from 'react';
import {Link, useNavigate} from 'react-router-dom';
import {Navbar, Container, Nav} from 'react-bootstrap';
import axios from 'axios';
import {AxiosConfig, BASE_URL} from '../components/AxiosConfig';
import {useCookies} from "react-cookie";
import CustomModal, {CustomModalTypes} from "./viewComponents/CustomModal";
import {
    authTokenExists,
    getUserRoles,
    removeAuthToken, removeUserRoles,
    setAuthToken, setUserRoles,
    userCookiePathConf,
    userIdCookieName, userRolesExists
} from "../components/AuthManager";
import {SettingsContext} from "../components/SettingsManager";
import ShowUserSavedArticles from "./user/ShowUserSavedArticles";

//main
function NavbarMenu() {

    const navigate = useNavigate();
    const [cookies, setCookie, removeCookie] = useCookies();
    const [user, setUser] = useState({
        data: null,
        error: null
    });
    const [modalSettings, setModalSettings] = useState({
        header: "",
        text: "",
        show: false,
        type: ""
    });
    const settings = useContext(SettingsContext);

    const [displayList, setDisplayList] = useState(false);

    function doLogout() {
        removeAuthToken();
        removeUserRoles();
        removeCookie(userIdCookieName, userCookiePathConf);
        setModalSettings({
            header: "Odhlášení",
            text: "Odhlášení proběhlo úspěšně.",
            type: CustomModalTypes.Success,
            show: true
        });
    }

    useEffect(() => {
        if(authTokenExists()) {
            axios.get(BASE_URL + "/auth/token", AxiosConfig()).then(res => {
                setAuthToken(res.data.body.token);
            }).catch(err => {
                removeAuthToken();
                removeCookie(userIdCookieName, userCookiePathConf);
                removeUserRoles();
                if (err.response.data.errors) {
                    setModalSettings({
                        header: "Chyba",
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        show: true,
                        type: CustomModalTypes.Danger
                    });
                }
            });
        }
        if(cookies[userIdCookieName]){
            axios.get(BASE_URL + '/user-management/current-user', AxiosConfig()).then(res => {
                setUser({
                    data: res.data.body,
                    error: res.data.errors,
                });

                if (res.data.body) {
                    if (Number(res.data.body.id) !== cookies.userId) {
                        setCookie(userIdCookieName, res.data.body.id, userCookiePathConf);
                        setUserRoles(res.data.body.roles);
                    }
                } else {
                    removeUserRoles();
                    removeCookie(userIdCookieName, userCookiePathConf);
                }
            }).catch(err => {
                if(err.response.data.key==="token-expired"){
                    console.log("TOKEN EXPIRED");
                    removeCookie(userIdCookieName, {path: '/'});
                    removeAuthToken();
                    removeUserRoles();
                    setModalSettings({
                        header: "Přihlášení",
                        text: err.response.data.message,
                        show: true,
                        type: CustomModalTypes.Warning
                    });
                } else if(err.response.data.errors){
                    setModalSettings({
                        header: "Chyba",
                        text: err.response.data.errors.map(e=>e.message).join(" "),
                        show: true,
                        type: CustomModalTypes.Danger
                    });
                } else {
                    removeUserRoles();
                    removeAuthToken();
                    removeCookie(userIdCookieName, {path:'/'});
                }
            });
        }
    }, [cookies[userIdCookieName], window.location.pathname]);

    function handleModalClose() {
        setModalSettings({
            header: modalSettings.header,
            text: modalSettings.text,
            show: false,
            type: CustomModalTypes.Success
        });
        navigate(0);
    }

    return (
        <>
            <CustomModal modalData={modalSettings} showModal={modalSettings.show} onClose={handleModalClose}/>
            <ShowUserSavedArticles show={displayList} onClose={() => setDisplayList(false)}/>
            <Navbar collapseOnSelect expand="lg" bg="dark" variant="dark">
                <Container>
                    <Navbar.Brand as={Link} to="/">{settings.blogName}</Navbar.Brand>
                    <Navbar.Toggle aria-controls="responsive-navbar-nav"/>
                    <Navbar.Collapse id="responsive-navbar-nav">
                        <Nav className="me-auto">
                            <Nav.Link as={Link} to="/clanky">Články</Nav.Link>
                            {settings.enableAuthorPage
                                ? <Nav.Link as={Link} to="/autor">{settings.authorPageLinkText}</Nav.Link>
                                : ""}
                            {settings.enableGalleryPage
                                ? <Nav.Link as={Link} to={"/galerie"}>{settings.galleryPageLinkText}</Nav.Link>
                                : ""}
                        </Nav>
                        {!user.data ? (
                            <Nav>
                                <Nav.Link as={Link} to="/prihlaseni">Přihlášení</Nav.Link>
                                <Nav.Link eventKey={2} as={Link} to="/registrace">Registrace</Nav.Link>
                            </Nav>
                        ) : (
                            <Nav>
                                <Navbar.Text>Přihlášen jako:
                                    <Link to={`/uzivatel/${user.data.id}`}>{user.data.username}</Link>
                                </Navbar.Text>
                                <Nav.Link onClick={() => setDisplayList(true)}>
                                    Uložené články
                                </Nav.Link>
                                <Nav.Link onClick={doLogout}>Odhlásit</Nav.Link>
                                {userRolesExists()
                                    ? (getUserRoles().includes("ADMIN")
                                        ? <Nav.Link href={"/admin"}>Admin sekce</Nav.Link>
                                        : "")
                                    : ""}
                            </Nav>
                        )}
                    </Navbar.Collapse>
                </Container>
            </Navbar>
        </>
    )
}

export default NavbarMenu;