import React, {useState, useEffect} from 'react';
import {useNavigate, useParams} from "react-router-dom";
import {Accordion, Button, Col, Form, Row, Table} from "react-bootstrap";
import ReactQuill from "react-quill";
import 'react-quill/dist/quill.snow.css';
import 'react-quill/dist/quill.bubble.css';
import {EditorFormats, EditorModules} from "./EditorSettings";
import ImageResize from "quill-image-resize-module-react/src/ImageResize";
import axios from "axios";
import {AxiosConfig, AxiosConfigImage, BASE_URL} from "../../../components/AxiosConfig";
import parse from 'html-react-parser';
import CustomAlert, {CustomAlertVariants} from "../../viewComponents/CustomAlert";
import CustomModal, {CustomModalTypes} from "../../viewComponents/CustomModal";
import {setWebTitle} from "../../../components/SettingsManager";
import ImageUpload from "../../viewComponents/ImageUpload";
import {ARTICLE_IMAGE_URL} from "../../../components/Constans";

ReactQuill.Quill.register('modules/ImageResize', ImageResize)

function ArticleEditor() {

    const articleId = useParams().articleId;
    const navigate = useNavigate();

    const [articleData, setArticleData] = useState({
        loading: true,
        title: "",
        descriptionType: "auto",
        description: "",
        text: "",
        visibility: true
    });
    const [containSurvey, setContainSurvey] = useState(false);
    const [visibleSurveyResult, setVisibleSurveyResult] = useState(false);
    const [surveyText, setSurveyText] = useState("");
    const [surveyOptions, setSurveyOptions] = useState([{
        position: 1,
        text: ""
    }]);
    const [descLength, setDescLength] = useState(0);
    const DESC_MAX_LENGTH = 250;

    const [displayAlert, setDisplayAlert] = useState(false);
    const [alertData, setAlertData] = useState({
        variant: "",
        header: "",
        content: ""
    });

    const [displayModal, setDisplayModal] = useState(false);
    const [modalData, setModalData] = useState({
        header: "",
        type: "",
        text: "",
        buttonText: ""
    });
    const [imageData, setImageData] = useState(null);

    useEffect(() => {
        if (articleId) {
            setWebTitle("Úprava článku " + articleData.title, true);
        } else {
            setWebTitle("Přidat článek", true)
        }
    }, [articleData]);

    useEffect(() => {
        if (articleId !== undefined) {
            axios.get(BASE_URL + "/content/article/" + articleId, AxiosConfig()).then(res => {
                setArticleData({
                    title: res.data.body.title,
                    text: res.data.body.text,
                    description: res.data.body.description,
                    visibility: true,
                    descriptionType: "manual"
                });
                setDescLength(res.data.body.description.length);
                if(res.data.body.survey){
                    setSurveyText(res.data.body.survey.surveyText);
                    setSurveyOptions(res.data.body.survey.surveyOptions.map(opt => {return{text: opt.text, position: opt.position}}));
                    setContainSurvey(true);
                    setVisibleSurveyResult(res.data.body.survey.surveyVisibleResults);
                } else {
                    setContainSurvey(false);
                }
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        header: "Chyba",
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        type: CustomModalTypes.Danger
                    });
                    setDisplayModal(true)
                }
            });
        }
    }, []);

    const handleDescChange = e => {
        setDescLength(Number(e.target.value.length));
        handleEditorChange(e);
    }

    const handleEditorChange = e => {
        const {target} = e;
        setArticleData((prevState => ({
            ...prevState,
            [target.name]: target.value
        })));
        setDescLength(articleData.description.length);
    }
    const handleProcedureContentChange = (content) => {
        if (articleData.descriptionType === "auto") {
            setArticleData((prevState => ({
                ...prevState,
                text: content,
                description: content.substring(0, 250).replace(/<[^>]*>?/gm, '')
            })));
        } else {
            setArticleData(prevState => ({
                ...prevState,
                text: content
            }));
        }
    };

    function handleSurveyOptionChange(e, index){
        const {value} = e.target;
        let newOptions = [...surveyOptions];
        newOptions[index].text = value;
        setSurveyOptions([...newOptions]);
    }

    function addSurveyOption(){
        if(!articleId) {
            setSurveyOptions((prevState => [...prevState, {position: surveyOptions.length + 1, text: ""}]));
        } else {
            setSurveyOptions((prevState => [...prevState,
                {position: surveyOptions[surveyOptions.length-1].position + 1, text: ""}]));
        }
    }

    function removeSurveyOption(index) {
        let newOptions = [...surveyOptions];
        newOptions.splice(index, 1);
        if(!articleId) {
            for (let i = index; i < newOptions.length; i++) {
                newOptions[i] = {position: (i + 1), text: newOptions[i].text};
            }
        }
        setSurveyOptions([...newOptions]);
    }

    const [loadSecondAccord, setLoadSecondAccord] = useState(false);

    function uploadArticle() {
        let articleDataPrepared = articleData;
        delete articleDataPrepared.descriptionType;
        delete articleDataPrepared.loading;
        if(containSurvey)
            articleDataPrepared.survey = {
                surveyText: surveyText,
                surveyOptions: surveyOptions,
                surveyVisibleResults: visibleSurveyResult
            }
        if(!containSurvey)
            articleDataPrepared.survey = null;
        let dataSend = new FormData();
        dataSend.append("articleData", new Blob([JSON.stringify(articleDataPrepared)], {type:"application/json"}));
        if(imageData)
            dataSend.append("articleImage", imageData);
        if (articleId) {
            axios.put(BASE_URL + "/content/article/" + Number(articleId), dataSend, AxiosConfigImage()).then(res => {
                setModalData({
                    type: CustomModalTypes.Success,
                    header: "Úprava článku",
                    text: res.data.body,
                    buttonText: "Zpět"
                });
                setDisplayModal(true);
            }).catch(err => {
                setAlertData({
                    variant: CustomAlertVariants.Danger,
                    header: "Úprava článku",
                    content: err.response.data.errors
                });
                setDisplayAlert(true);
            });
        } else {
            axios.post(BASE_URL + "/content/article/", dataSend, AxiosConfigImage()).then(res => {
                setModalData({
                    type: CustomModalTypes.Success,
                    header: "Přidání článku",
                    text: res.data.body,
                    buttonText: "Zpět"
                });
                setDisplayModal(true);
            }).catch(err => {
                setAlertData({
                    variant: CustomAlertVariants.Danger,
                    header: "Přidání článku",
                    content: err.response.data.errors
                });
                setDisplayAlert(true);
            });
        }
    }

    return (
        <div id={"content"}>
            <CustomAlert alertData={alertData} display={displayAlert} onCloseAlert={() => setDisplayAlert(false)}/>
            <CustomModal modalData={modalData} showModal={displayModal} onClose={() => {
                navigate(-1)
            }}/>
            <h1>{articleId ? "Upravit článek " : "Přidat článek"}</h1>
            {articleId && articleData.loading ? "loading" :
                <Accordion defaultActiveKey={0}>
                    <Accordion.Item eventKey={0}>
                        <Accordion.Header>Editor článku</Accordion.Header>
                        <Accordion.Body>
                            <Form>
                                <Form.Group className="mb-3">
                                    <Form.Label>Název článku</Form.Label>
                                    <Form.Control placeholder={"Název článku"}
                                                  type={"text"}
                                                  name={"title"}
                                                  onChange={handleEditorChange}
                                                  value={articleData.title}/>
                                </Form.Group>
                                <Row className={"article-editor-info-row"}>
                                    <Form.Group className={"mb-3"} as={Col}>
                                        <Form.Label>Vyberte způsob tvorby stručného popis:</Form.Label>
                                        <Form.Select aria-label={"Vyberte způsob tvorby stručného popis"}
                                                     value={articleData.descriptionType}
                                                     name={"descriptionType"}
                                                     onChange={handleEditorChange}>
                                            <option value="manual">Vlastní popis článku</option>
                                            <option value="auto">Automatický popis článku</option>
                                        </Form.Select>
                                    </Form.Group>
                                    <Form.Group as={Col}>
                                        <Form.Label>Viditelnost článku:</Form.Label>
                                        <Form.Select aria-label={"Viditelnost článku"}
                                                     value={articleData.visibility}
                                                     name={"visibility"}
                                                     onChange={handleEditorChange}>
                                            <option value={true}>Zobrazit článek</option>
                                            <option value={false}>Skrýt článek</option>
                                        </Form.Select>
                                    </Form.Group>
                                    <Col>
                                        <Form.Label>Nahrajte obrázek.</Form.Label>
                                        <br/>
                                        <ImageUpload showPreview={true}
                                                     displayName={true}
                                                     onChangeSuccess={(e) => setImageData(e)}
                                                     onChangeError={(err) => {
                                                         setAlertData({
                                                             header: "Chyba",
                                                             content: err,
                                                             variant: CustomAlertVariants.Danger
                                                         });
                                                         setDisplayAlert(true);
                                                     }}/>
                                        {articleId
                                            ? <>
                                                <br/>
                                                Aktuální obrázek:
                                                <br/>
                                                <img src={ARTICLE_IMAGE_URL(articleId)}
                                                     alt={""} width={200} height={200}/>
                                            </>
                                            : ""}
                                    </Col>
                                </Row>
                                <Form.Group className={"mb-3"}>
                                    <Form.Label>Stručný popis článku.
                                        {articleData.descriptionType === "manual"
                                            ? " Počet znaků: " + descLength + "/" + DESC_MAX_LENGTH
                                            : ""}
                                    </Form.Label>
                                    <Form.Control value={articleData.description} placeholder={"Stručný popis článku"}
                                                  disabled={articleData.descriptionType === "auto"}
                                                  type={"text"} as={"textarea"}
                                                  maxLength={DESC_MAX_LENGTH}
                                                  style={{maxHeight: "120px", minHeight: "120px"}}
                                                  name={"description"}
                                                  onChange={handleDescChange}/>
                                </Form.Group>
                                <Form.Group>
                                    <Form.Label>Text článku</Form.Label>
                                    <ReactQuill
                                        value={articleData.text}
                                        className={"editor-admin"}
                                        theme="snow"
                                        modules={EditorModules}
                                        formats={EditorFormats}
                                        onChange={handleProcedureContentChange}>
                                    </ReactQuill>
                                </Form.Group>
                                <br/>
                                <Form.Group>
                                    <Form.Label className={"text-center"} as={"h3"}>Anketa k článku</Form.Label>
                                    <Row>
                                        <Form.Switch checked={containSurvey}
                                                     className={"d-flex justify-content-center"}
                                                     onChange={() => setContainSurvey(!containSurvey)}
                                                     label={"Má tento článek obsahovat anketu."}/>
                                        <Form.Switch checked={visibleSurveyResult}
                                                     className={"d-flex justify-content-center"}
                                                     onChange={() => setVisibleSurveyResult(!visibleSurveyResult)}
                                                     label={"Zobrazit výsledky ankety pro čtenáře?"}/>
                                    </Row>
                                    <Row>
                                        <Col className={"text-center"}>
                                            <Form.Label>Text k anketě:</Form.Label>
                                            <Form.Control type={"text"}
                                                          value={surveyText}
                                                          onChange={(e) => setSurveyText(e.target.value)}
                                                          placeholder={"Napište název / text k anketě..."}/>
                                        </Col>
                                    </Row>
                                    <Row className={"justify-content-center align-items-center"} onClick={addSurveyOption}>
                                        <Button variant={"outline-success"} className={"w-auto"}>Přidat další možnost</Button>
                                    </Row>
                                    <Table responsive className={"article-editor-survey-table"}>
                                        <tbody>
                                        {surveyOptions ? surveyOptions.map((option, index) => {
                                            return (
                                                <tr key={index}>
                                                    <td>Pozice: {option.position}</td>
                                                    <td>Odpověď:</td>
                                                    <td>
                                                        <Form.Control type={"text"} value={option.text} onChange={(e) => handleSurveyOptionChange(e, index)}/>
                                                    </td>
                                                    <td className={"text-center"}>
                                                        <Button variant={"outline-danger"}
                                                                onClick={() => removeSurveyOption(index)}>Odebrat možnost</Button>
                                                    </td>
                                                </tr>
                                            )
                                        }) : ""}
                                        </tbody>
                                    </Table>
                                </Form.Group>
                                <br/>
                                <Row className={"justify-content-center"}>
                                <Button variant={"success"}
                                        className={"w-50"}
                                        onClick={uploadArticle}>Uložit</Button>
                                </Row>
                            </Form>
                        </Accordion.Body>
                    </Accordion.Item>
                    <Accordion.Item eventKey={1}>
                        <Accordion.Header>Ukázka článku</Accordion.Header>
                        <Accordion.Body onEnter={() => setLoadSecondAccord(true)}
                                        onExited={() => setLoadSecondAccord(false)}>
                            {loadSecondAccord ?
                                <div>
                                    <h3>{articleData.title}</h3>
                                    <p>{articleData.description}</p>
                                    <div className={"ql-editor"}>{parse(articleData.text)}</div>
                                    {/**
                                     <ReactQuill value={articleData.text} readOnly={true} theme={"bubble"}/>
                                     **/}
                                </div> : ""}
                        </Accordion.Body>
                    </Accordion.Item>
                </Accordion>
            }
        </div>
    )
}

export default ArticleEditor;