import React, {useEffect, useState} from "react";
import {Accordion, Button, Col, Container, Form, Row} from "react-bootstrap";
import {AxiosConfig, AxiosConfigImage, BASE_URL} from "../../../components/AxiosConfig";
import axios from "axios";
import CustomModal, {CustomModalTypes} from "../../viewComponents/CustomModal";
import Lightbox from "yet-another-react-lightbox";
import PhotoAlbum from "react-photo-album";
import {Counter, Fullscreen, Slideshow, Thumbnails} from "yet-another-react-lightbox/plugins";
import "yet-another-react-lightbox/plugins/thumbnails.css";
import "yet-another-react-lightbox/styles.css";
import "yet-another-react-lightbox/plugins/counter.css";


function GallerySettingsAdmin() {

    const [images, setImages] = useState(undefined);
    const [imageUrls, setImageUrls] = useState([]);
    const [disableUploadButton, setDisableUploadButton] = useState(false);
    const [modalData, setModalData] = useState({
        text: "",
        header: "",
        show: false,
        type: CustomModalTypes.Success
    });
    const [savedImageUrls, setSavedImageUrls] = useState(undefined);
    const [index, setIndex] = useState(-1);
    const [galleryError, setGalleryError] = useState(undefined);

    useEffect(() => {
        if (images) {
            setDisableUploadButton(false);
            let urls = [];
            images.forEach(image => {
                urls.push(URL.createObjectURL(image));
            });
            setImageUrls([...urls]);
        } else {
            setDisableUploadButton(true);
        }
    }, [images]);

    useEffect(() => {
        axios.get(BASE_URL + "/settings/gallery/image/list", AxiosConfig()).then(res => {
            setSavedImageUrls([...res.data.body.map(n => {
                return {
                    name: n.name,
                    src: imageUlrFromName(n.name),
                    width: n.width,
                    height: n.height
                }
            })]);
            setGalleryError(undefined);
        }).catch(err => {
            if (err.response.data.errors)
                setGalleryError(err.response.data.errors.map(e => e.message).join(" "));
        });
    }, []);

    function imageInputChange(e) {
        setImages([...e.target.files]);
    }

    function displayImages() {
        let rows = [];
        let elements = [];
        const perRow = 5;
        imageUrls.forEach((url, index) => {
            if (index % perRow === 0) {
                rows.push(<Row className={"mb-2 align-items-end"}>{elements}</Row>);
                elements = [];
            }
            elements.push(
                <Col className={"gallery-image-col"} key={index}>
                    <img className={"gallery-image"}
                         alt={index + " test"}
                         src={url}/>
                    <Button variant={"outline-danger"} onClick={() => removeImage(index)}>Smazat</Button>
                </Col>
            );
            if (index === (imageUrls.length - 1)) {
                let missingElements = perRow - elements.length;
                for (let i = 0; i < missingElements; i++) {
                    elements.push(<Col className={"gallery-image-col"}/>)
                }
                rows.push(<Row className={"mb-2 align-items-end"}>{elements}</Row>);
            }
        });
        return rows;
    }

    function imageUlrFromName(name) {
        return BASE_URL + "/settings/gallery/image/" + name;
    }

    function removeImage(indexToDelete) {
        let tempArr = [];
        images.forEach((image, index) => {
            if (index !== indexToDelete)
                tempArr.push(image);
        });
        setImages([...tempArr]);
    }

    function submitPhotos() {
        setDisableUploadButton(true);
        let formData = new FormData();
        images.forEach(image => {
            formData.append("files", image);
        })
        axios.post(BASE_URL + "/settings/gallery/upload", formData, AxiosConfigImage()).then(res => {
            setImages(undefined);
            setImageUrls(undefined);
            setDisableUploadButton(false);
            setModalData({
                show: true,
                header: "Nahrání fotografií",
                text: res.data.body,
                type: CustomModalTypes.Success
            });
        }).catch(err => {
            setModalData({
                show: true,
                header: "Nahrání fotografií",
                text: err.response.data.errors ? err.response.data.errors.map(e => e.message).join(" ") : "Došlo k neznámé chybě.",
                type: CustomModalTypes.Danger
            });
        })
    }

    function deleteSavedImage(name) {
        axios.delete(BASE_URL + "/settings/gallery/image/" + name, AxiosConfig()).then(res => {
            setModalData({
                header: "Odebrání obrázku",
                text: res.data.body,
                show: true,
                type: CustomModalTypes.Success
            });
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Odebrání obrázku",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
    }

    return (
        <div id={"content"}>
            <CustomModal modalData={modalData} showModal={modalData.show} onClose={() => setModalData({show: false})}/>
            <h1>Nastavení galerie</h1>
            <Accordion defaultActiveKey={[""]} alwaysOpen={true}>
                <Accordion.Item eventKey={"0"}>
                    <Accordion.Header>Přidání nových fotografií</Accordion.Header>
                    <Accordion.Body>
                        <Form>
                            <Row>
                                <Col className={"text-center align-self-center"}>
                                    <Form.Label>Fotografie</Form.Label>
                                    <Form.Control type={"file"}
                                                  className={"w-auto mx-auto"}
                                                  multiple={true}
                                                  accept={"image/png, image/jpeg, image/gif"}
                                                  onChange={imageInputChange}/>
                                    <Button className={"w-auto mx-auto"}
                                            variant={"outline-success"}
                                            onClick={submitPhotos}
                                            disabled={disableUploadButton}>Nahrát nové fotografie</Button>
                                </Col>
                            </Row>
                            <Row>
                            </Row>
                        </Form>
                        {imageUrls && imageUrls.length > 0 ? <Container>{displayImages()}</Container> : ""}
                    </Accordion.Body>
                </Accordion.Item>
                <Accordion.Item eventKey={"1"}>
                    <Accordion.Header>Smazat fotografie</Accordion.Header>
                    <Accordion.Body>
                        <Row>
                            {savedImageUrls ? savedImageUrls.map((image, index) => {
                                return (
                                    <Col key={index}>
                                        <img height={150} src={image.src} alt={image.name}/>
                                        <br/>
                                        <Button variant={"outline-danger"}
                                                className={"w-auto"}
                                                onClick={() => deleteSavedImage(image.name)}>Smazat</Button>
                                    </Col>
                                )
                            }) : ""}
                        </Row>
                    </Accordion.Body>
                </Accordion.Item>
                <Accordion.Item eventKey={"2"}>
                    <Accordion.Header>Aktuální fotografie</Accordion.Header>
                    <Accordion.Body>
                        {savedImageUrls
                            ? <>
                                <PhotoAlbum photos={savedImageUrls}
                                            layout="rows"
                                            targetRowHeight={150}
                                            onClick={({index}) => setIndex(index)}/>
                                <Lightbox index={index}
                                          slides={savedImageUrls}
                                          open={index >= 0}
                                          close={() => setIndex(-1)}
                                          plugins={[Fullscreen, Slideshow, Thumbnails, Counter]}/>
                            </> : <h3 className={"text-center"}>{galleryError}</h3>}
                    </Accordion.Body>
                </Accordion.Item>
            </Accordion>
        </div>
    )
}

export default GallerySettingsAdmin;