import React, {useState, useEffect, useContext} from 'react'
import 'bootstrap';
import axios from 'axios';
import {Button, Card, CardGroup, Col, Container, Form, Row, Spinner} from 'react-bootstrap';
import {AxiosConfig, BASE_URL} from '../../components/AxiosConfig';
import {formatDateTimeWithSettings, SettingsContext, setWebTitle} from "../../components/SettingsManager";
import {Link} from "react-router-dom";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";
import {ARTICLE_IMAGE_URL} from "../../components/Constans";
import CustomPagination from "../viewComponents/CustomPagination";

function Articles() {

    const [articles, setArticles] = useState(undefined);
    const [searchArticle, setSearchArticle] = useState("");

    const settings = useContext(SettingsContext);

    const [messageData, setMessageData] = useState({
        hidden: true,
        text: ""
    });

    const [page, setPage] = useState(1);
    const perPage = 10;

    useEffect(() => {
        setWebTitle("Články", true);
        defaultLoadArticles();
    }, []);

    function defaultLoadArticles(){
        setSearchArticle("");
        axios.get(BASE_URL + '/content/article/list', AxiosConfig()).then(res => {
            setArticles(res.data.body);
            setMessageData({hidden: true, text: ""});
        }).catch(err => {
            setArticles(null);
            if(err.response.data.errors)
                setMessageData({hidden: false, text: err.response.data.errors.map(e=>e.message).join(" ")})
        });
    }

    function doSearch(e) {
        e.preventDefault();
        if(searchArticle) {
            let config = {...AxiosConfig()};
            config.params = {search: searchArticle};
            axios.get(BASE_URL + "/content/article/search", config).then(res => {
                setArticles(res.data.body);
                setMessageData({hidden: true, text: ""});
            }).catch(err => {
                setArticles(null);
                if(err.response.data.errors)
                    setMessageData({hidden: false, text: err.response.data.errors.map(e=>e.message).join(" ")})
            });
        }
    }

    return (
        <div id="content">
            <SEOManager description={"Zobrazení všech článků blogu "+ settings.blogName}
                        robots={SEO_ROBOTS.INDEX_FOLLOW}/>
            <Form onSubmit={doSearch}>
                <Row className={"align-items-center mt-2 mb-2 articles-search-row"}>
                    <Form.Control type={"text"}
                                  placeholder={"Vyhledávání"}
                                  className={"w-25 ms-auto"}
                                  value={searchArticle}
                                  onChange={(e) => setSearchArticle(e.target.value)}/>
                    <Button type={"submit"}
                            variant={"outline-success"}
                            className={"ms-1 w-auto"}>Vyhledat</Button>
                    <Button variant={"outline-danger"}
                            onClick={defaultLoadArticles}
                            className={"ms-1 me-auto w-auto"}>Zrušit</Button>
                </Row>
            </Form>
            <h4 hidden={messageData.hidden} className={"text-center"}>
                {messageData.text}
            </h4>
            <Container>
                {articles ?
                    articles.slice(((page - 1) * perPage), (page * perPage)).map((article, index) => {
                        return (
                                <Row key={index} className={"border mb-2 article-preview-row"}>
                                    <Col className={"d-flex align-items-center justify-content-center article-image-col"}>
                                        <img src={ARTICLE_IMAGE_URL(article.id)}
                                             alt={"Obrázek pro " + article.title}/>
                                    </Col>
                                    <Col className={"ms-2 d-flex flex-column flex-wrap"}>
                                    <Row>
                                        <Col>
                                            <Link to={"/clanek/"+article.id}>
                                                <h5>{article.title}</h5>
                                            </Link>
                                        </Col>
                                    </Row>
                                    <Row>
                                        <p>
                                        {article.description}
                                        </p>
                                    </Row>
                                    <Row className={"align-items-end justify-content-end"}>
                                        <Col className={"text-start"}>
                                            {settings.displayUpdateCreateTime !== "update"
                                                ? "Napsáno dne: "+formatDateTimeWithSettings(article.createTime)
                                                : ""}
                                        </Col>
                                        <Col className={"text-end"}>
                                            {article.createTime !== article.lastUpdateTime
                                                && settings.displayUpdateCreateTime !== "create"
                                                ? "Poslední úprava: "+formatDateTimeWithSettings(article.lastUpdateTime)
                                                : ""}
                                        </Col>
                                    </Row>
                                    </Col>
                                </Row>
                        );
                    }) : messageData.hidden ? <Spinner/> : ""}
                <Row>
                    {articles ?
                    <CustomPagination numOfItems={articles.length}
                                      defaultPage={1}
                                      maxItemsPerPage={perPage}
                                      reloadDependency={articles}
                                      onChange={(newPage) => setPage(newPage)}/>
                        : "" }
                </Row>
            </Container>
        </div>
    )

}


export default Articles