import {Button, Col, Form, Row} from "react-bootstrap";
import {useEffect, useState} from "react";
import axios from "axios";
import {AxiosConfigImage, BASE_URL} from "../../components/AxiosConfig";
import {useNavigate} from 'react-router-dom';
import CustomModal, {CustomModalTypes} from "../viewComponents/CustomModal";
import CustomAlert, {CustomAlertVariants} from "../viewComponents/CustomAlert";
import ImageUpload from "../viewComponents/ImageUpload";
import {setWebTitle} from "../../components/SettingsManager";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function Registration(){

    const navigate = useNavigate();
    const [validated, setValidated] = useState(false);
    const [regFormData, setRegFormData] = useState({
        username: "",
        email: "",
        password: "",
        confirmPassword: ""
    });
    const [regFormImage, setRegFormImage] = useState({
        imageData: null,
        valid: false
    });
    const [alertData, setAlertData] = useState({
        content:"",
        variant:"",
        header:""
    });
    const [hiddenError, setHiddenError] = useState(true);
    const [modalData, setModalData] = useState({
        header: "",
        text: "",
        type: CustomModalTypes.Success,
        show: false
    });

    useEffect(() => {
        setWebTitle("Registrace", true);
    }, []);

    function registrationSubmit(e){
        e.preventDefault();
        setValidated(true);
        if(e.currentTarget.checkValidity() === false) {
            e.stopPropagation();
        } else {
            let dataToSend = new FormData();
            dataToSend.append("userData", new Blob([JSON.stringify(regFormData)], {type:"application/json"}));
            if(regFormImage.valid)
                dataToSend.append("image", regFormImage.imageData);
            axios.post(BASE_URL + "/auth/registration", dataToSend, AxiosConfigImage()).then(res => {
                setHiddenError(true);
                if(res.data.errors){
                    setModalData({
                        header: "Registrace téměř úspěšná",
                        text: res.data.body + " " + res.data.errors.map(e => e.message).join(" "),
                        type: CustomModalTypes.Success,
                        show: true
                    });
                } else {
                    setModalData({
                        header: "Registrace úspěšná",
                        text: res.data.body,
                        type: CustomModalTypes.Success,
                        show: true
                    });
                }
            }).catch(err => {
                setValidated(false);
                setAlertData({
                    content: err.response.data.errors,
                    header: "Chyby při registraci",
                    variant: CustomAlertVariants.Danger
                });
                setHiddenError(false);
            });
        }
    }

    function handleChangeForm(e){
        const {name, value} = e.target;
        let state = {};
        for(const [key, currentValue] of Object.entries(regFormData)){
            if(key === name){
                state[key] = value;
            } else {
                state[key] = currentValue;
            }
        }
        setRegFormData(state);
    }

    function handleCloseModal(){
        setModalData({show: false});
        navigate('/prihlaseni');
    }

    return (
      <div id='content'>
          <SEOManager description={"Registrace do systému"}
                      robots={SEO_ROBOTS.INDEX_FOLLOW}/>
          <CustomModal
              modalData={modalData}
              showModal={modalData.show}
              onClose={handleCloseModal}/>
          <CustomAlert
              alertData={alertData}
              display={!hiddenError} onCloseAlert={() => setHiddenError(true)}/>
          <Form noValidate validated={validated} onSubmit={registrationSubmit}>
              <Form.Text as="h2">Povinné údaje</Form.Text>
              <Row>
                  <Form.Group as={Col} className='mb-3'>
                      <Form.Label>Uživatelské jméno:</Form.Label>
                      <Form.Control required type="text" placeholder="Zadejte uživatelské jméno"
                                    name="username" minLength={1} maxLength={32}
                                    value={regFormData.username} onChange={handleChangeForm}/>
                      <Form.Control.Feedback type="invalid">Uživatelské jméno musí být zadáno.</Form.Control.Feedback>
                      <Form.Control.Feedback type="valid">Uživatelské je zadáno.</Form.Control.Feedback>
                  </Form.Group>
                  <Form.Group as={Col} className='mb-3'>
                      <Form.Label>Email:</Form.Label>
                      <Form.Control required type="mail" placeholder="Zadejte email"
                                    name="email" minLength={5} maxLength={70}
                                    value={regFormData.email} onChange={handleChangeForm}/>
                      <Form.Control.Feedback type="invalid">Email musí být zadán.</Form.Control.Feedback>
                      <Form.Control.Feedback type="valid">Email je zadán.</Form.Control.Feedback>
                  </Form.Group>
              </Row>
              <Row>
                  <Form.Group as={Col} className="mb-3">
                      <Form.Label>Heslo:</Form.Label>
                      <Form.Control required type="password" placeholder="Zadejte heslo."
                                    name="password" minLength={4}
                                    value={regFormData.password} onChange={handleChangeForm}/>
                      <Form.Control.Feedback type="invalid">Heslo musí být zadáno.</Form.Control.Feedback>
                      <Form.Control.Feedback type="valid">Heslo je zadáno.</Form.Control.Feedback>
                  </Form.Group>
                  <Form.Group as={Col} className="mb-3">
                      <Form.Label>Potvrdit heslo: </Form.Label>
                      <Form.Control required type="password" placeholder="Potvrďte heslo"
                                    name="confirmPassword" minLength={4}
                                    value={regFormData.confirmPassword} onChange={handleChangeForm}/>
                      <Form.Control.Feedback type="invalid">Potvrzující heslo musí být zadáno.</Form.Control.Feedback>
                      <Form.Control.Feedback type="valid">Potvrzující heslo je zadáno.</Form.Control.Feedback>
                  </Form.Group>
              </Row>
              <Form.Text>Nepovinné údaje</Form.Text>
              <Row>
                  <Form.Group as={Col} className={"mb-3 border"}>
                  <ImageUpload
                      showPreview={true}
                      displayName={true}
                      displayHint={true}
                      onChangeError={(error) => {
                          setAlertData({
                              content: error,
                              header: "Chyba při načítání obrázku",
                              variant: CustomAlertVariants.Danger
                          });
                          setRegFormImage({
                              imageData: null,
                              valid: false
                          });
                          setHiddenError(false);
                      }}
                      onChangeSuccess={(e) => {
                          setRegFormImage({
                              imageData: e,
                              valid: true
                          });
                          setHiddenError(true);
                      }}/>
                  </Form.Group>
                  <Form.Group as={Col}></Form.Group>
              </Row>
              <Button variant="primary" type="submit">Registrovat</Button>
          </Form>
      </div>
    );
}

export default Registration;