import axios from 'axios';
import React, {useEffect, useState} from 'react'
import {AxiosConfig, BASE_URL} from '../../components/AxiosConfig';
import {Link, useNavigate, useParams} from "react-router-dom";
import {Accordion, Button, Card, Col, Dropdown, DropdownButton, Row, Stack} from "react-bootstrap";
import {useCookies} from "react-cookie";
import StarRatings from "react-star-ratings/build/star-ratings";
import {ARTICLE_IMAGE_URL, getGender, USER_IMAGE_URL} from "../../components/Constans";
import {formatDateTimeWithSettings, formatDateWithSettings, setWebTitle} from "../../components/SettingsManager";
import CustomModal, {CustomModalTypes, CustomYesNoModal} from "../viewComponents/CustomModal";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";
import {userRolesExists} from "../../components/AuthManager";
import moment from "moment";

function ShowProfile() {
    const userId = useParams().userId;
    const [cookies] = useCookies("userId");
    const [myProfile, setMyProfile] = useState(false);
    const [userData, setUserData] = useState({
        username: "",
        id: "",
        email: "",
        registrationTime: "",
        imageUrl: "",
        birthDate: "",
        gender: "",
        bio: ""
    });

    const [userRatingData, setUserRatingData] = useState([{
        id: null,
        title: null,
        description: null,
        rating: null,
        createTime: null
    }]);

    const [userCommentData, setUserCommentData] = useState([{
        commentId: null,
        articleId: null,
        title: null,
        description: null,
        comment: null,
        createTime: null
    }]);
    const [modalData, setModalData] = useState({});
    const [yesNoModalData, setYesNoModalData] = useState({
        show:false,
        header: "",
        text: ""
    });
    const navigate = useNavigate();

    useEffect(() => {
        setWebTitle(userData.username, true);
    }, [userData]);

    useEffect(() => {
        axios.get(BASE_URL + "/user-management/user/" + userId, AxiosConfig()).then(res => {
            let data = {...res.data.body};
            data.imageUrl = USER_IMAGE_URL(userId);
            if (res.status === 206 && res.data.errors.length > 0) {
                let nullKeys = [];
                for (let key in data) {
                    if (data[key] == null) nullKeys.push(key);
                }
                res.data.errors.forEach((err) => {
                    nullKeys.forEach(key => {
                        if(err.key.replace("privacy-", "").includes("-")) {
                            let parts = err.key.replace("privacy-", "").split("-");
                            let finalKey = parts[0] + parts[1].charAt(0).toUpperCase() + parts[1].slice(1);
                            if(finalKey === key){
                                data[key] = err.message;
                            }
                        }
                        if ("privacy-" + key === err.key) {
                            data[key] = err.message;
                        }
                    })
                });
            }
            setUserData(data);
            setMyProfile((Number)(cookies.userId) === (Number)(res.data.body.id));
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
        axios.get(BASE_URL + "/user-management/user/" + userId + "/rating", AxiosConfig()).then(res => {
            setUserRatingData(res.data.body);
        }).catch(err => {
            setUserRatingData({error: err.response.data.errors.map(e => e.message).join(" ")});
        });
        loadCommentData();
    }, [userId]);

    function loadCommentData(){
        axios.get(BASE_URL + "/user-management/user/" + userId + "/comment", AxiosConfig()).then(res => {
            setUserCommentData(res.data.body);
        }).catch(err => {
            setUserCommentData({error: err.response.data.errors.map(e => e.message).join(" ")});
        });
    }

    function deleteComment(e) {
        let deleteId = e.target.attributes.commentid.value;
        axios.delete(BASE_URL + '/content/article/comment/' + deleteId, AxiosConfig()).then(res => {
            setModalData({
                show: true,
                header: "Smazání komentáře",
                text: res.data.body,
                type: "success"
            });
            loadCommentData();
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
    }

    function setCommentIncorrect(e){
        let commentId = Number(e.target.attributes.commentid.value);
        console.log(commentId);
        let incorrect = userCommentData.find(comment => commentId === comment.commentId).incorrect;
        setYesNoModalData({
            show:true,
            text: "Chcete tento komentář skrýt jako nevhodný? Komentář bude možné i nadále zobrazit, bude nutné však zobrazení potvrdit.",
            header: "Skrytí nevhodného komentáře",
            commentId: commentId,
            incorrect: incorrect
        });
    }

    function confirmIncorrectComment(){
        let commentId = yesNoModalData.commentId;
        let incorrect = yesNoModalData.incorrect;
        setYesNoModalData({show:false});
        axios.put(BASE_URL + "/content/article/comment/"+commentId+"/incorrect", (!incorrect), AxiosConfig()).then(res => {
            setModalData({
                show: true,
                type: CustomModalTypes.Success,
                text: res.data.body,
                header: "Úprava viditelnosti komentáře."
            });
            loadCommentData();
        }).catch(err => {
            if(err.response.data.errors){
                setModalData({
                    show:true,
                    type: CustomModalTypes.Danger,
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    header: "Úprava viditelnosti komentáře."
                })
            }
        })
    }


    return (
        <div id='content'>
            <SEOManager description={"Uživatel"+ userData.username + " " + userData.bio}
                        robots={SEO_ROBOTS.INDEX_FOLLOW}/>
            <CustomModal modalData={modalData} showModal={modalData.show} onClose={() => setModalData({show:false})}/>
            <CustomYesNoModal modalData={yesNoModalData}
                              showModal={yesNoModalData.show}
                              onConfirm={confirmIncorrectComment}
                              onCancel={()=>setYesNoModalData({show:false})}/>

            <Card className={"mt-5"}>
                <Stack className={"show-user"} direction={"horizontal"} gap={5}>
                    <div>
                        <img src={userData.imageUrl}
                             alt={" preview"}
                             width={250}
                             height={250}/>
                    </div>
                    <div>
                        <h4>{userData.username}</h4>
                        <p>Email: {userData.email}</p>
                        <p>Registrace:
                            {moment(userData.registrationTime).isValid()
                                ? " " + formatDateTimeWithSettings(userData.registrationTime)
                                : " " + userData.registrationTime}
                        </p>
                        {userData.birthDate
                            ? <p>Datum narození:
                                {moment(userData.birthDate).isValid()
                                    ? " " + formatDateWithSettings(userData.birthDate)
                                    : " " + userData.birthDate}
                        </p>
                            : ""}
                        {userData.gender
                            ? <p>Pohlaví: {userData.gender ? getGender(userData.gender) : ""}</p>
                            : ""}
                    </div>
                    <div>
                        <Card.Text style={{whiteSpace: "pre-line"}}>{userData.bio}</Card.Text>
                    </div>
                    <div className={"ms-auto me-2 mt-2 align-self-start"}>
                        {myProfile
                            ? <Button variant={"outline-dark"}
                                      onClick={() => navigate("/upravit-profil")}>Upravit profil</Button>
                            : ""}
                        {userRolesExists("ADMIN")
                            ?<DropdownButton title={"Možnosti admina"} variant={"outline-info"}>
                                <Dropdown.Item href={"/admin/statistiky/uzivatel/"+userId}>Zobrazit statistiky</Dropdown.Item>
                            </DropdownButton>
                            : ""}
                    </div>
                </Stack>
            </Card>
            <Accordion defaultActiveKey={['']} alwaysOpen>
                <Accordion.Item eventKey={'0'}>
                    <Accordion.Header>{!userRatingData.error ? "Hodnocení uživatele" : userRatingData.error}</Accordion.Header>
                    {!userRatingData.error ?
                        <Accordion.Body className={"p-0"}>
                            {userRatingData.map((rating, index) => {
                                return (

                                    <div key={index + "-article-rating-" + rating.id}
                                         className={"border-bottom border-1 p-3"}>
                                        <div>
                                            <Row className={"show-user-rating-comment-row mb-2"}>
                                                <Col style={{maxWidth: "40px"}} className={"me-1"}>
                                                    <img src={USER_IMAGE_URL(userData.id)}
                                                         alt={userData.username + " profile image"}
                                                         height={40} width={40}/>
                                                </Col>
                                                <Col className={"d-flex align-items-center justify-content-start"}>
                                                    <Link to={"#"}>{userData.username}</Link>
                                                    <span className={"ps-1"}>hodnotil: </span>
                                                    <span className={"align-self-start ms-2"}>
                                                        <StarRatings
                                                            rating={rating.rating ? rating.rating : 0}
                                                            starDimension={"20px"}
                                                            starSpacing={"1px"}
                                                            starRatedColor={"green"}/>
                                                    </span>
                                                </Col>
                                                <Col className={"t-end end d-flex align-items-center justify-content-end"}>
                                                    <span>{formatDateTimeWithSettings(rating.createTime)}</span>
                                                </Col>
                                            </Row>
                                        </div>
                                        <div className={"border-1 border"}>
                                            <Row className={"show-user-article-preview-row"}>
                                                <Col className={"d-flex align-items-center justify-content-center show-article-image"}>
                                                    <img src={ARTICLE_IMAGE_URL(rating.id)} alt={"Obrázek pro " + rating.title} />
                                                </Col>
                                                <Col>
                                            <Row className={"m-1"}>
                                                <Link to={"/clanek/"+rating.id}>{rating.title}</Link>
                                            </Row>
                                            <Row className={"ms-2"}>
                                                <p>
                                                    {rating.description}
                                                </p>
                                            </Row>
                                                </Col>
                                            </Row>
                                        </div>
                                    </div>
                                )
                            })}
                        </Accordion.Body> : ""}
                </Accordion.Item>
                <Accordion.Item eventKey={'1'}>
                    <Accordion.Header>{!userCommentData.error ? "Komentáře uživatele" : userCommentData.error}</Accordion.Header>
                    {!userCommentData.error ?
                        <Accordion.Body className={"p-0"}>
                            {userCommentData.map((comment, index) => {
                                return (
                                    <div key={index + "-article-comment-" + comment.commentId}
                                         className={"border-bottom border-1 p-3"}>
                                        <div>
                                            <Row className={"mb-2 show-user-rating-comment-row"}>
                                                <Col style={{maxWidth: "40px"}} className={"me-1"}>
                                                    <img src={USER_IMAGE_URL(userData.id)}
                                                         alt={userData.username + " profile image"}
                                                         height={40} width={40}/>
                                                </Col>
                                                <Col className={"d-flex align-items-center justify-content-start"}>
                                                    <Link to={"#"}>{userData.username}</Link>
                                                    <span className={"ps-1"}>komentoval: </span>
                                                </Col>
                                                <Col
                                                    className={"t-end d-flex align-items-center justify-content-end"}>
                                                    <span>{formatDateTimeWithSettings(comment.createTime)}</span>
                                                    {Number(cookies.userId) === Number(comment.userId)
                                                        ? <Button commentid={comment.commentId}
                                                                  className={"ms-3"}
                                                                  variant={"outline-danger"}
                                                                  onClick={deleteComment}>Smazat</Button>
                                                        : ""}
                                                    {userRolesExists("ADMIN")
                                                        ? <Button commentid={comment.commentId}
                                                                  className={"ms-3 w-auto"}
                                                                  variant={comment.incorrect ? "success" : "danger"}
                                                                  onClick={setCommentIncorrect}>
                                                            {comment.incorrect ? "Vhodný" : "Nevhodný"}
                                                        </Button>
                                                        : ""}
                                                </Col>
                                            </Row>
                                            <Row className={"ms-4 mb-2"} id={"comment-article-id-"+comment.commentId}>
                                                {comment.incorrect
                                                    ? <div><i>Komentář je správcem označen jako nevhodný. </i>
                                                        <Link onClick={() => {
                                                                    document.getElementById("comment-article-id-"+comment.commentId).innerText = comment.comment;
                                                                }}>
                                                            <i>Zobrazit</i>
                                                        </Link>
                                                    </div>
                                                    : comment.comment}
                                            </Row>
                                        </div>
                                        <div className={"border-1 border"}>
                                            <Row className={"show-user-article-preview-row"}>
                                            <Col className={"d-flex align-items-center justify-content-center show-article-image"}>
                                                <img src={ARTICLE_IMAGE_URL(comment.articleId)} alt={"Obrázek pro " + comment.title} />
                                            </Col>
                                            <Col>
                                            <Row className={"m-1"}>
                                                <Link to={"/clanek/"+comment.articleId}>{comment.title}</Link>
                                            </Row>
                                            <Row className={"ms-2"}>
                                                <p>
                                                    {comment.description}
                                                </p>
                                            </Row>
                                            </Col>
                                            </Row>
                                        </div>
                                    </div>
                                )
                            })}
                        </Accordion.Body> : ""}
                </Accordion.Item>
            </Accordion>
        </div>
    );
}


export default ShowProfile;